@@ -54,6 +54,8 @@ typedef enum {
  * @GS_APP_LIST_FILTER_FLAG_KEY_ID:		Filter by ID
  * @GS_APP_LIST_FILTER_FLAG_KEY_SOURCE:		Filter by default source
  * @GS_APP_LIST_FILTER_FLAG_KEY_VERSION:	Filter by version
+ * @GS_APP_LIST_FILTER_FLAG_PREFER_INSTALLED:	Prefer installed applications
+ * @GS_APP_LIST_FILTER_FLAG_KEY_ID_PROVIDES:	Filter using the provides ID
  *
  * Flags to use when filtering. The priority of eash #GsApp is used to choose
  * which application object to keep.
@@ -63,6 +65,8 @@ typedef enum {
 	GS_APP_LIST_FILTER_FLAG_KEY_ID		= 1 << 0,
 	GS_APP_LIST_FILTER_FLAG_KEY_SOURCE	= 1 << 1,
 	GS_APP_LIST_FILTER_FLAG_KEY_VERSION	= 1 << 2,
+	GS_APP_LIST_FILTER_FLAG_PREFER_INSTALLED= 1 << 3,
+	GS_APP_LIST_FILTER_FLAG_KEY_ID_PROVIDES	= 1 << 4,
 	/*< private >*/
 	GS_APP_LIST_FILTER_FLAG_LAST
 } GsAppListFilterFlags;
