@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2013-2018 Richard Hughes <richard@hughsie.com>
- * Copyright (C) 2014-2015 Kalev Lember <klember@redhat.com>
+ * Copyright (C) 2014-2018 Kalev Lember <klember@redhat.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -57,7 +57,6 @@ struct _GsAppClass
  * @GS_APP_KUDO_USES_APP_MENU:		Uses an AppMenu for navigation
  * @GS_APP_KUDO_HAS_SCREENSHOTS:	Supplies screenshots
  * @GS_APP_KUDO_POPULAR:		Is popular
- * @GS_APP_KUDO_PERFECT_SCREENSHOTS:	Supplies perfect screenshots
  * @GS_APP_KUDO_HIGH_CONTRAST:		Installs a high contrast icon
  * @GS_APP_KUDO_HI_DPI_ICON:		Installs a HiDPI icon
  * @GS_APP_KUDO_SANDBOXED:		Application is sandboxed
@@ -77,7 +76,6 @@ typedef enum {
 	GS_APP_KUDO_USES_APP_MENU		= 1 << 8,
 	GS_APP_KUDO_HAS_SCREENSHOTS		= 1 << 9,
 	GS_APP_KUDO_POPULAR			= 1 << 10,
-	GS_APP_KUDO_PERFECT_SCREENSHOTS		= 1 << 12,
 	GS_APP_KUDO_HIGH_CONTRAST		= 1 << 13,
 	GS_APP_KUDO_HI_DPI_ICON			= 1 << 14,
 	GS_APP_KUDO_SANDBOXED			= 1 << 15,
@@ -299,9 +297,6 @@ void		 gs_app_add_category		(GsApp		*app,
 						 const gchar	*category);
 gboolean	 gs_app_remove_category		(GsApp		*app,
 						 const gchar	*category);
-GPtrArray	*gs_app_get_keywords		(GsApp		*app);
-void		 gs_app_set_keywords		(GsApp		*app,
-						 GPtrArray	*keywords);
 void		 gs_app_add_kudo		(GsApp		*app,
 						 GsAppKudo	 kudo);
 void		 gs_app_remove_kudo		(GsApp		*app,
@@ -325,6 +320,8 @@ void		 gs_app_remove_quirk		(GsApp		*app,
 						 AsAppQuirk	 quirk);
 gboolean	 gs_app_is_installed		(GsApp		*app);
 gboolean	 gs_app_is_updatable		(GsApp		*app);
+gchar		*gs_app_get_origin_ui		(GsApp		*app);
+gchar		*gs_app_get_packaging_format	(GsApp		*app);
 G_END_DECLS
 
 #endif /* __GS_APP_H */
