@@ -24,7 +24,7 @@
 
 #include <glib-object.h>
 
-#include "gs-app-list.h"
+#include "gs-app-list-private.h"
 #include "gs-auth.h"
 #include "gs-category.h"
 #include "gs-plugin-types.h"
@@ -40,6 +40,8 @@ void		 gs_plugin_job_set_refine_flags		(GsPluginJob	*self,
 							 GsPluginRefineFlags refine_flags);
 void		 gs_plugin_job_set_filter_flags		(GsPluginJob	*self,
 							 GsPluginRefineFlags filter_flags);
+void		 gs_plugin_job_set_dedupe_flags		(GsPluginJob	*self,
+							 GsAppListFilterFlags dedupe_flags);
 void		 gs_plugin_job_set_interactive		(GsPluginJob	*self,
 							 gboolean	 interactive);
 void		 gs_plugin_job_set_max_results		(GsPluginJob	*self,
