@@ -1763,6 +1763,8 @@ gs_plugin_action_to_function_name (GsPluginAction action)
 		return "gs_plugin_destroy";
 	if (action == GS_PLUGIN_ACTION_PURCHASE)
 		return "gs_plugin_app_purchase";
+	if (action == GS_PLUGIN_ACTION_GET_ALTERNATES)
+		return "gs_plugin_add_alternates";
 	return NULL;
 }
 
@@ -1865,6 +1867,8 @@ gs_plugin_action_to_string (GsPluginAction action)
 		return "destroy";
 	if (action == GS_PLUGIN_ACTION_PURCHASE)
 		return "purchase";
+	if (action == GS_PLUGIN_ACTION_GET_ALTERNATES)
+		return "get-alternates";
 	return NULL;
 }
 
@@ -1967,6 +1971,8 @@ gs_plugin_action_from_string (const gchar *action)
 		return GS_PLUGIN_ACTION_DESTROY;
 	if (g_strcmp0 (action, "purchase") == 0)
 		return GS_PLUGIN_ACTION_PURCHASE;
+	if (g_strcmp0 (action, "get-alternates") == 0)
+		return GS_PLUGIN_ACTION_GET_ALTERNATES;
 	return GS_PLUGIN_ACTION_UNKNOWN;
 }
 
@@ -2036,6 +2042,16 @@ gs_plugin_refine_flags_to_string (GsPluginRefineFlags refine_flags)
 		g_ptr_array_add (cstrs, "require-runtime");
 	if (refine_flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_SCREENSHOTS)
 		g_ptr_array_add (cstrs, "require-screenshots");
+	if (refine_flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_CATEGORIES)
+		g_ptr_array_add (cstrs, "require-categories");
+	if (refine_flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_PROJECT_GROUP)
+		g_ptr_array_add (cstrs, "require-project-group");
+	if (refine_flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_DEVELOPER_NAME)
+		g_ptr_array_add (cstrs, "require-developer-name");
+	if (refine_flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_KUDOS)
+		g_ptr_array_add (cstrs, "require-kudos");
+	if (refine_flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_CONTENT_RATING)
+		g_ptr_array_add (cstrs, "content-rating");
 	if (cstrs->len == 0)
 		return g_strdup ("none");
 	g_ptr_array_add (cstrs, NULL);
