@@ -144,38 +144,22 @@ gs_utils_cache_func (void)
 static void
 gs_utils_error_func (void)
 {
-	g_autofree gchar *app_id = NULL;
-	g_autofree gchar *origin_id = NULL;
+	guint i;
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GsApp) app = gs_app_new ("gimp.desktop");
-	g_autoptr(GsApp) origin = gs_app_new ("gimp-repo");
 
-	for (guint i = 0; i < GS_PLUGIN_ERROR_LAST; i++)
+	for (i = 0; i < GS_PLUGIN_ERROR_LAST; i++)
 		g_assert (gs_plugin_error_to_string (i) != NULL);
 
-	/* noop */
-	gs_utils_error_add_app_id (&error, app);
-	gs_utils_error_add_origin_id (&error, origin);
-
+	gs_utils_error_add_unique_id (&error, app);
 	g_set_error (&error,
 		     GS_PLUGIN_ERROR,
 		     GS_PLUGIN_ERROR_DOWNLOAD_FAILED,
 		     "failed");
 	g_assert_cmpstr (error->message, ==, "failed");
-	gs_utils_error_add_app_id (&error, app);
-	gs_utils_error_add_origin_id (&error, origin);
-	g_assert_cmpstr (error->message, ==, "[*/*/*/*/gimp-repo/*] {*/*/*/*/gimp.desktop/*} failed");
-
-	/* find and strip any unique IDs from the error message */
-	for (guint i = 0; i < 2; i++) {
-		if (app_id == NULL)
-			app_id = gs_utils_error_strip_app_id (error);
-		if (origin_id == NULL)
-			origin_id = gs_utils_error_strip_origin_id (error);
-	}
-
-	g_assert_cmpstr (app_id, ==, "*/*/*/*/gimp.desktop/*");
-	g_assert_cmpstr (origin_id, ==, "*/*/*/*/gimp-repo/*");
+	gs_utils_error_add_unique_id (&error, app);
+	g_assert_cmpstr (error->message, ==, "[*/*/*/*/gimp.desktop/*] failed");
+	gs_utils_error_strip_unique_id (error);
 	g_assert_cmpstr (error->message, ==, "failed");
 }
 
@@ -287,6 +271,7 @@ gs_plugin_func (void)
 	GsAppList *list_dup;
 	GsAppList *list_remove;
 	GsApp *app;
+	g_autoptr(AsProvide) prov = as_provide_new ();
 
 	/* check enums converted */
 	for (guint i = 0; i < GS_PLUGIN_ACTION_LAST; i++) {
@@ -437,6 +422,48 @@ gs_plugin_func (void)
 	g_assert_cmpstr (gs_app_get_unique_id (gs_app_list_index (list, 0)), ==, "user/foo/repo-security/*/*/*");
 	g_object_unref (list);
 
+	/* prefer installed applications */
+	list = gs_app_list_new ();
+	app = gs_app_new ("e");
+	gs_app_set_state (app, AS_APP_STATE_INSTALLED);
+	gs_app_set_unique_id (app, "user/foo/*/*/e/*");
+	gs_app_set_priority (app, 0);
+	gs_app_list_add (list, app);
+	g_object_unref (app);
+	app = gs_app_new ("e");
+	gs_app_set_state (app, AS_APP_STATE_AVAILABLE);
+	gs_app_set_unique_id (app, "user/bar/*/*/e/*");
+	gs_app_set_priority (app, 100);
+	gs_app_list_add (list, app);
+	g_object_unref (app);
+	gs_app_list_filter_duplicates (list,
+				       GS_APP_LIST_FILTER_FLAG_KEY_ID |
+				       GS_APP_LIST_FILTER_FLAG_PREFER_INSTALLED);
+	g_assert_cmpint (gs_app_list_length (list), ==, 1);
+	g_assert_cmpstr (gs_app_get_unique_id (gs_app_list_index (list, 0)), ==, "user/foo/*/*/e/*");
+	g_object_unref (list);
+
+	/* use the provides ID to dedupe */
+	list = gs_app_list_new ();
+	app = gs_app_new ("gimp.desktop");
+	gs_app_set_unique_id (app, "user/fedora/*/*/gimp.desktop/*");
+	gs_app_set_priority (app, 0);
+	gs_app_list_add (list, app);
+	g_object_unref (app);
+	app = gs_app_new ("org.gimp.GIMP");
+	as_provide_set_kind (prov, AS_PROVIDE_KIND_ID);
+	as_provide_set_value (prov, "gimp.desktop");
+	gs_app_add_provide (app, prov);
+	gs_app_set_unique_id (app, "user/flathub/*/*/org.gimp.GIMP/*");
+	gs_app_set_priority (app, 100);
+	gs_app_list_add (list, app);
+	g_object_unref (app);
+	gs_app_list_filter_duplicates (list, GS_APP_LIST_FILTER_FLAG_KEY_ID_PROVIDES);
+	g_assert_cmpint (gs_app_list_length (list), ==, 1);
+	g_assert_cmpstr (gs_app_get_unique_id (gs_app_list_index (list, 0)), ==,
+			 "user/flathub/*/*/org.gimp.GIMP/*");
+	g_object_unref (list);
+
 	/* use globs when adding */
 	list = gs_app_list_new ();
 	app = gs_app_new ("b");
@@ -476,6 +503,19 @@ gs_plugin_func (void)
 	g_assert_cmpint (gs_app_list_length (list), ==, 2);
 	g_object_unref (list);
 
+	/* allow duplicating a wildcard */
+	list = gs_app_list_new ();
+	app = gs_app_new ("gimp.desktop");
+	gs_app_add_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX);
+	gs_app_list_add (list, app);
+	g_object_unref (app);
+	app = gs_app_new ("gimp.desktop");
+	gs_app_add_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX);
+	gs_app_list_add (list, app);
+	g_object_unref (app);
+	g_assert_cmpint (gs_app_list_length (list), ==, 1);
+	g_object_unref (list);
+
 	/* add a list to a list */
 	list = gs_app_list_new ();
 	list_dup = gs_app_list_new ();
