@@ -51,16 +51,12 @@ static GHashTable *
 gs_plugin_appstream_create_app_hash (AsStore *store)
 {
 	GHashTable *hash;
+	GPtrArray *apps;
 	guint i;
-	g_autoptr(GPtrArray) apps = NULL;
 
 	hash = g_hash_table_new_full (g_str_hash, g_str_equal,
 				      g_free, (GDestroyNotify) g_object_unref);
-#if AS_CHECK_VERSION(0,7,15)
-	apps = as_store_dup_apps (store);
-#else
-	apps = g_ptr_array_ref (as_store_get_apps (store));
-#endif
+	apps = as_store_get_apps (store);
 	for (i = 0; i < apps->len; i++) {
 		AsApp *app = g_ptr_array_index (apps, i);
 		gchar *key = g_strdup (as_app_get_id (app));
@@ -251,6 +247,7 @@ gboolean
 gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
+	GPtrArray *items;
 	gboolean ret;
 	const gchar *tmp;
 	const gchar *test_xml;
@@ -259,7 +256,6 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 	guint *perc;
 	guint i;
 	g_autoptr(GHashTable) origins = NULL;
-	g_autoptr(GPtrArray) items = NULL;
 
 	/* Parse the XML */
 	if (g_getenv ("GNOME_SOFTWARE_PREFER_LOCAL") != NULL) {
@@ -290,11 +286,7 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 			return FALSE;
 		}
 	}
-#if AS_CHECK_VERSION(0,7,15)
-	items = as_store_dup_apps (priv->store);
-#else
-	items = g_ptr_array_ref (as_store_get_apps (priv->store));
-#endif
+	items = as_store_get_apps (priv->store);
 	if (items->len == 0) {
 		g_warning ("No AppStream data, try 'make install-sample-data' in data/");
 		g_set_error (error,
@@ -364,6 +356,48 @@ gs_plugin_url_to_app (GsPlugin *plugin,
 	return TRUE;
 }
 
+static void
+gs_plugin_appstream_set_compulsory_quirk (GsApp *app, AsApp *item)
+{
+	GPtrArray *array;
+	const gchar *current_desktop;
+
+	/*
+	 * Set the core applications for the current desktop that cannot be
+	 * removed.
+	 *
+	 * If XDG_CURRENT_DESKTOP contains ":", indicating that it is made up
+	 * of multiple components per the Desktop Entry Specification, an app
+	 * is compulsory if any of the components in XDG_CURRENT_DESKTOP match
+	 * any value in <compulsory_for_desktops />. In that way,
+	 * "GNOME-Classic:GNOME" shares compulsory apps with GNOME.
+	 *
+	 * As a special case, if the <compulsory_for_desktop /> value contains
+	 * a ":", we match the entire XDG_CURRENT_DESKTOP. This lets people set
+	 * compulsory apps for such compound desktops if they want.
+	 *
+	 */
+	array = as_app_get_compulsory_for_desktops (item);
+	current_desktop = g_getenv ("XDG_CURRENT_DESKTOP");
+	if (current_desktop != NULL) {
+		g_auto(GStrv) xdg_current_desktops = g_strsplit (current_desktop, ":", 0);
+		for (guint i = 0; i < array->len; i++) {
+			const gchar *tmp = g_ptr_array_index (array, i);
+			/* if the value has a :, check the whole string */
+			if (g_strstr_len (tmp, -1, ":")) {
+				if (g_strcmp0 (current_desktop, tmp) == 0) {
+					gs_app_add_quirk (app, AS_APP_QUIRK_COMPULSORY);
+					break;
+				}
+			/* otherwise check if any element matches this one */
+			} else if (g_strv_contains ((const gchar * const *) xdg_current_desktops, tmp)) {
+				gs_app_add_quirk (app, AS_APP_QUIRK_COMPULSORY);
+				break;
+			}
+		}
+	}
+}
+
 static gboolean
 gs_plugin_refine_from_id (GsPlugin *plugin,
 			  GsApp *app,
@@ -387,15 +421,8 @@ gs_plugin_refine_from_id (GsPlugin *plugin,
 	item = as_store_get_app_by_unique_id (priv->store, unique_id,
 					      AS_STORE_SEARCH_FLAG_USE_WILDCARDS);
 	if (item == NULL) {
-		g_autoptr(GPtrArray) apps = NULL;
-		g_autoptr(GPtrArray) apps_merge = NULL;
-
+		GPtrArray *apps = as_store_get_apps (priv->store);
 		g_debug ("no app with ID %s found in system appstream", unique_id);
-#if AS_CHECK_VERSION(0,7,15)
-		apps = as_store_dup_apps (priv->store);
-#else
-		apps = g_ptr_array_ref (as_store_get_apps (priv->store));
-#endif
 		for (guint i = 0; i < apps->len; i++) {
 			item = g_ptr_array_index (apps, i);
 			if (g_strcmp0 (as_app_get_id (item), gs_app_get_id (app)) != 0)
@@ -405,24 +432,15 @@ gs_plugin_refine_from_id (GsPlugin *plugin,
 		}
 
 		/* fall back to trying to get a merge app */
-#if AS_CHECK_VERSION(0,7,15)
-		apps_merge = as_store_dup_apps_by_id_merge (priv->store, gs_app_get_id (app));
-		for (guint i = 0; i < apps_merge->len; i++) {
-			item = g_ptr_array_index (apps_merge, i);
-			if (!gs_appstream_refine_app (plugin, app, item, error))
-				return FALSE;
-		}
-#else
-		apps_merge = as_store_get_apps_by_id_merge (priv->store, gs_app_get_id (app));
-		if (apps_merge != NULL) {
-			g_ptr_array_ref (apps_merge);
-			for (guint i = 0; i < apps_merge->len; i++) {
-				item = g_ptr_array_index (apps_merge, i);
+		apps = as_store_get_apps_by_id_merge (priv->store, gs_app_get_id (app));
+		if (apps != NULL) {
+			for (guint i = 0; i < apps->len; i++) {
+				item = g_ptr_array_index (apps, i);
 				if (!gs_appstream_refine_app (plugin, app, item, error))
 					return FALSE;
+				gs_plugin_appstream_set_compulsory_quirk (app, item);
 			}
 		}
-#endif
 		return TRUE;
 	}
 
@@ -468,15 +486,11 @@ gs_plugin_add_distro_upgrades (GsPlugin *plugin,
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	AsApp *item;
+	GPtrArray *array;
 	guint i;
-	g_autoptr(GPtrArray) array = NULL;
 
 	/* find any upgrades */
-#if AS_CHECK_VERSION(0,7,15)
-	array = as_store_dup_apps (priv->store);
-#else
-	array = g_ptr_array_ref (as_store_get_apps (priv->store));
-#endif
+	array = as_store_get_apps (priv->store);
 	for (i = 0; i < array->len; i++) {
 		g_autoptr(GsApp) app = NULL;
 		item = g_ptr_array_index (array, i);
@@ -503,6 +517,13 @@ gs_plugin_refine_app (GsPlugin *plugin,
 {
 	gboolean found = FALSE;
 
+	/* not us */
+	if (gs_app_get_bundle_kind (app) != AS_BUNDLE_KIND_PACKAGE &&
+	    gs_app_get_bundle_kind (app) != AS_BUNDLE_KIND_UNKNOWN) {
+		g_debug ("not a package, ignoring");
+		return TRUE;
+	}
+
 	/* find by ID then package name */
 	if (!gs_plugin_refine_from_id (plugin, app, &found, error))
 		return FALSE;
@@ -557,11 +578,10 @@ gs_plugin_refine_wildcard (GsPlugin *plugin,
 
 		/* does the app have an installation method */
 		if (as_app_get_pkgname_default (item) == NULL &&
-		    as_app_get_bundle_default (item) == NULL &&
-		    as_app_get_metadata_item (item, "shell-extensions::uuid") == NULL) {
+		    as_app_get_bundle_default (item) == NULL) {
 			g_debug ("not using %s for wildcard as "
-				 "no installation method",
-				 as_app_get_unique_id (item));
+				 "no bundle or pkgname",
+				 as_app_get_id (item));
 			continue;
 		}
 
@@ -617,14 +637,10 @@ gs_plugin_add_installed (GsPlugin *plugin,
 			 GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
-	g_autoptr(GPtrArray) array = NULL;
+	GPtrArray *array;
 
 	/* search categories for the search term */
-#if AS_CHECK_VERSION(0,7,15)
-	array = as_store_dup_apps (priv->store);
-#else
-	array = g_ptr_array_ref (as_store_get_apps (priv->store));
-#endif
+	array = as_store_get_apps (priv->store);
 	for (guint i = 0; i < array->len; i++) {
 		AsApp *item = g_ptr_array_index (array, i);
 		if (as_app_get_state (item) == AS_APP_STATE_INSTALLED) {
@@ -683,6 +699,17 @@ gs_plugin_add_recent (GsPlugin *plugin,
 					cancellable, error);
 }
 
+gboolean
+gs_plugin_add_alternates (GsPlugin *plugin,
+			  GsApp *app,
+			  GsAppList *list,
+			  GCancellable *cancellable,
+			  GError **error)
+{
+	GsPluginData *priv = gs_plugin_get_data (plugin);
+	return gs_appstream_add_alternates (plugin, priv->store, app, list, cancellable, error);
+}
+
 gboolean
 gs_plugin_refresh (GsPlugin *plugin,
 		   guint cache_age,
