@@ -280,6 +280,20 @@ gs_plugin_dummy_timeout_add (guint timeout_ms, GCancellable *cancellable)
 	g_main_loop_run (helper->loop);
 }
 
+gboolean
+gs_plugin_add_alternates (GsPlugin *plugin,
+			  GsApp *app,
+			  GsAppList *list,
+			  GCancellable *cancellable,
+			  GError **error)
+{
+	if (g_strcmp0 (gs_app_get_id (app), "zeus.desktop") == 0) {
+		g_autoptr(GsApp) app2 = gs_app_new ("chiron.desktop");
+		gs_app_list_add (list, app2);
+	}
+	return TRUE;
+}
+
 gboolean
 gs_plugin_add_search (GsPlugin *plugin,
 		      gchar **values,
@@ -588,7 +602,7 @@ gs_plugin_update_app (GsPlugin *plugin,
 				     GS_PLUGIN_ERROR,
 				     GS_PLUGIN_ERROR_DOWNLOAD_FAILED,
 				     "no network connection is available");
-		gs_utils_error_add_origin_id (error, priv->cached_origin);
+		gs_utils_error_add_unique_id (error, priv->cached_origin);
 		return FALSE;
 	}
 
