@@ -95,12 +95,12 @@ gs_plugins_dummy_install_func (GsPluginLoader *plugin_loader)
 static void
 gs_plugins_dummy_error_func (GsPluginLoader *plugin_loader)
 {
+	GsPluginEvent *event;
 	const GError *app_error;
 	gboolean ret;
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GPtrArray) events = NULL;
 	g_autoptr(GsApp) app = NULL;
-	g_autoptr(GsPluginEvent) event = NULL;
 	g_autoptr(GsPluginJob) plugin_job = NULL;
 
 	/* drop all caches */
@@ -190,7 +190,6 @@ gs_plugins_dummy_metadata_quirks (GsPluginLoader *plugin_loader)
 
 	gs_app_set_metadata (app, "GnomeSoftware::quirks::not-launchable", "true");
 
-	g_object_unref (plugin_job);
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_REFINE,
 					 "app", app,
 					 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_DESCRIPTION,
@@ -205,7 +204,6 @@ gs_plugins_dummy_metadata_quirks (GsPluginLoader *plugin_loader)
 	gs_app_set_metadata (app, "GnomeSoftware::quirks::not-launchable", NULL);
 	gs_app_set_metadata (app, "GnomeSoftware::quirks::not-launchable", "false");
 
-	g_object_unref (plugin_job);
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_REFINE,
 					 "app", app,
 					 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_DESCRIPTION,
@@ -354,7 +352,6 @@ gs_plugins_dummy_installed_func (GsPluginLoader *plugin_loader)
 	GsApp *app;
 	GsApp *addon;
 	GsAppList *addons;
-	guint64 kudos;
 	g_autofree gchar *menu_path = NULL;
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GsAppList) list = NULL;
@@ -365,8 +362,10 @@ gs_plugins_dummy_installed_func (GsPluginLoader *plugin_loader)
 					 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_ORIGIN |
 							 GS_PLUGIN_REFINE_FLAGS_REQUIRE_ADDONS |
 							 GS_PLUGIN_REFINE_FLAGS_REQUIRE_LICENSE |
+							 GS_PLUGIN_REFINE_FLAGS_REQUIRE_KUDOS |
 							 GS_PLUGIN_REFINE_FLAGS_REQUIRE_MENU_PATH |
 							 GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
+							 GS_PLUGIN_REFINE_FLAGS_REQUIRE_CATEGORIES |
 							 GS_PLUGIN_REFINE_FLAGS_REQUIRE_PROVENANCE,
 					 NULL);
 	list = gs_plugin_loader_job_process (plugin_loader, plugin_job, NULL, &error);
@@ -390,8 +389,7 @@ gs_plugins_dummy_installed_func (GsPluginLoader *plugin_loader)
 	g_assert (gs_app_get_license_is_free (app));
 
 	/* check kudos */
-	kudos = gs_app_get_kudos (app);
-	g_assert (kudos & GS_APP_KUDO_MY_LANGUAGE);
+	g_assert_true (gs_app_has_kudo (app, GS_APP_KUDO_MY_LANGUAGE));
 
 	/* check categories */
 	g_assert (gs_app_has_category (app, "Player"));
@@ -441,6 +439,36 @@ gs_plugins_dummy_search_func (GsPluginLoader *plugin_loader)
 	g_assert_cmpint (gs_app_get_kind (app), ==, AS_APP_KIND_DESKTOP);
 }
 
+static void
+gs_plugins_dummy_search_alternate_func (GsPluginLoader *plugin_loader)
+{
+	GsApp *app_tmp;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GsAppList) list = NULL;
+	g_autoptr(GsApp) app = NULL;
+	g_autoptr(GsPluginJob) plugin_job = NULL;
+
+	/* get search result based on addon keyword */
+	app = gs_app_new ("zeus.desktop");
+	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_GET_ALTERNATES,
+					 "app", app,
+					 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON,
+					 NULL);
+	list = gs_plugin_loader_job_process (plugin_loader, plugin_job, NULL, &error);
+	gs_test_flush_main_context ();
+	g_assert_no_error (error);
+	g_assert (list != NULL);
+
+	/* make sure there is the original app, and the alternate */
+	g_assert_cmpint (gs_app_list_length (list), ==, 2);
+	app_tmp = gs_app_list_index (list, 0);
+	g_assert_cmpstr (gs_app_get_id (app_tmp), ==, "chiron.desktop");
+	g_assert_cmpint (gs_app_get_kind (app_tmp), ==, AS_APP_KIND_DESKTOP);
+	app_tmp = gs_app_list_index (list, 1);
+	g_assert_cmpstr (gs_app_get_id (app_tmp), ==, "zeus.desktop");
+	g_assert_cmpint (gs_app_get_kind (app_tmp), ==, AS_APP_KIND_DESKTOP);
+}
+
 static void
 gs_plugins_dummy_hang_func (GsPluginLoader *plugin_loader)
 {
@@ -910,6 +938,9 @@ main (int argc, char **argv)
 	g_test_add_data_func ("/gnome-software/plugins/dummy/search",
 			      plugin_loader,
 			      (GTestDataFunc) gs_plugins_dummy_search_func);
+	g_test_add_data_func ("/gnome-software/plugins/dummy/search-alternate",
+			      plugin_loader,
+			      (GTestDataFunc) gs_plugins_dummy_search_alternate_func);
 	g_test_add_data_func ("/gnome-software/plugins/dummy/hang",
 			      plugin_loader,
 			      (GTestDataFunc) gs_plugins_dummy_hang_func);
