@@ -220,7 +220,7 @@ _refresh_cache (GsPlugin *plugin,
 				      priv->cachefn,
 				      cancellable,
 				      error)) {
-		gs_utils_error_add_origin_id (error, priv->cached_origin);
+		gs_utils_error_add_unique_id (error, priv->cached_origin);
 		return FALSE;
 	}
 
@@ -301,10 +301,11 @@ _create_upgrade_from_info (GsPlugin *plugin, PkgdbItem *item)
 	app = gs_app_new (app_id);
 	gs_app_set_state (app, AS_APP_STATE_AVAILABLE);
 	gs_app_set_kind (app, AS_APP_KIND_OS_UPGRADE);
+	gs_app_set_bundle_kind (app, AS_BUNDLE_KIND_PACKAGE);
 	gs_app_set_name (app, GS_APP_QUALITY_LOWEST, item->name);
 	gs_app_set_summary (app, GS_APP_QUALITY_LOWEST,
 			    /* TRANSLATORS: this is a title for Fedora distro upgrades */
-			    _("Upgrade your Fedora system to the latest features and improvements."));
+			    _("Upgrade for the latest features, performance and stability improvements."));
 	gs_app_set_version (app, app_version);
 	gs_app_set_size_installed (app, 1024 * 1024 * 1024); /* estimate */
 	gs_app_set_size_download (app, 256 * 1024 * 1024); /* estimate */
