@@ -141,18 +141,11 @@ gs_appstream_refine_add_addons (GsPlugin *plugin,
 				GError **error)
 {
 	GPtrArray *addons;
-	g_autoptr(AsProfileTask) ptask = NULL;
 
 	/* we only care about addons to desktop apps */
 	if (gs_app_get_kind (app) != AS_APP_KIND_DESKTOP)
 		return TRUE;
 
-	/* search categories for the search term */
-	ptask = as_profile_start (gs_plugin_get_profile (plugin),
-				  "appstream::refine-addons{%s}",
-				  gs_app_get_unique_id (app));
-	g_assert (ptask != NULL);
-
 	addons = as_app_get_addons (item);
 	if (addons == NULL)
 		return TRUE;
@@ -250,44 +243,6 @@ gs_appstream_is_recent_release (AsApp *app)
 	return secs / (60 * 60 * 24) < 365;
 }
 
-static gboolean
-gs_appstream_are_screenshots_perfect (AsApp *app)
-{
-	AsImage *image;
-	AsScreenshot *screenshot;
-	GPtrArray *screenshots;
-	guint height;
-	guint width;
-
-	screenshots = as_app_get_screenshots (app);
-	if (screenshots->len == 0)
-		return FALSE;
-	for (guint i = 0; i < screenshots->len; i++) {
-
-		/* get the source image as the thumbs will be resized & padded */
-		screenshot = g_ptr_array_index (screenshots, i);
-		image = as_screenshot_get_source (screenshot);
-		if (image == NULL)
-			return FALSE;
-
-		width = as_image_get_width (image);
-		height = as_image_get_height (image);
-
-		/* too small */
-		if (width < AS_IMAGE_LARGE_WIDTH || height < AS_IMAGE_LARGE_HEIGHT)
-			return FALSE;
-
-		/* too large */
-		if (width > AS_IMAGE_LARGE_WIDTH * 2 || height > AS_IMAGE_LARGE_HEIGHT * 2)
-			return FALSE;
-
-		/* not 16:9 */
-		if ((width / 16) * 9 != height)
-			return FALSE;
-	}
-	return TRUE;
-}
-
 static void
 gs_appstream_copy_metadata (GsApp *app, AsApp *item)
 {
@@ -486,15 +441,7 @@ gs_appstream_refine_app (GsPlugin *plugin,
 	GPtrArray *array;
 	GPtrArray *pkgnames;
 	GPtrArray *kudos;
-	const gchar *current_desktop;
 	const gchar *tmp;
-	g_autoptr(AsProfileTask) ptask = NULL;
-
-	/* search categories for the search term */
-	ptask = as_profile_start (gs_plugin_get_profile (plugin),
-				  "appstream::refine-app{%s}",
-				  gs_app_get_unique_id (app));
-	g_assert (ptask != NULL);
 
 	/* set the kind to be more precise */
 	if (gs_app_get_kind (app) == AS_APP_KIND_UNKNOWN ||
@@ -647,11 +594,8 @@ gs_appstream_refine_app (GsPlugin *plugin,
 				    as_app_get_project_license (item));
 
 	/* set keywords */
-	if (as_app_get_keywords (item, NULL) != NULL &&
-	    gs_app_get_keywords (app) == NULL) {
-		gs_app_set_keywords (app, as_app_get_keywords (item, NULL));
+	if (as_app_get_keywords (item, NULL) != NULL)
 		gs_app_add_kudo (app, GS_APP_KUDO_HAS_KEYWORDS);
-	}
 
 	/* set origin */
 	if (as_app_get_origin (item) != NULL &&
@@ -701,42 +645,6 @@ gs_appstream_refine_app (GsPlugin *plugin,
 	    as_app_get_developer_name (item, NULL) != NULL)
 		gs_app_set_developer_name (app, as_app_get_developer_name (item, NULL));
 
-	/*
-	 * Set the core applications for the current desktop that cannot be
-	 * removed.
-	 *
-	 * If XDG_CURRENT_DESKTOP contains ":", indicating that it is made up
-	 * of multiple components per the Desktop Entry Specification, an app
-	 * is compulsory if any of the components in XDG_CURRENT_DESKTOP match
-	 * any value in <compulsory_for_desktops />. In that way,
-	 * "GNOME-Classic:GNOME" shares compulsory apps with GNOME.
-	 *
-	 * As a special case, if the <compulsory_for_desktop /> value contains
-	 * a ":", we match the entire XDG_CURRENT_DESKTOP. This lets people set
-	 * compulsory apps for such compound desktops if they want.
-	 *
-	 */
-	array = as_app_get_compulsory_for_desktops (item);
-	current_desktop = g_getenv ("XDG_CURRENT_DESKTOP");
-	if (current_desktop != NULL) {
-		g_auto(GStrv) xdg_current_desktops = g_strsplit (current_desktop, ":", 0);
-		for (guint i = 0; i < array->len; i++) {
-			tmp = g_ptr_array_index (array, i);
-			/* if the value has a :, check the whole string */
-			if (g_strstr_len (tmp, -1, ":")) {
-				if (g_strcmp0 (current_desktop, tmp) == 0) {
-					gs_app_add_quirk (app, AS_APP_QUIRK_COMPULSORY);
-					break;
-				}
-			/* otherwise check if any element matches this one */
-			} else if (g_strv_contains ((const gchar * const *) xdg_current_desktops,
-				   tmp)) {
-				gs_app_add_quirk (app, AS_APP_QUIRK_COMPULSORY);
-				break;
-			}
-		}
-	}
-
 	/* set id kind */
 	if (gs_app_get_kind (app) == AS_APP_KIND_UNKNOWN)
 		gs_app_set_kind (app, as_app_get_kind (item));
@@ -762,10 +670,6 @@ gs_appstream_refine_app (GsPlugin *plugin,
 	/* set provides */
 	gs_appstream_refine_add_provides (app, item);
 
-	/* are the screenshots perfect */
-	if (gs_appstream_are_screenshots_perfect (item))
-		gs_app_add_kudo (app, GS_APP_KUDO_PERFECT_SCREENSHOTS);
-
 	/* was this application released recently */
 	if (gs_appstream_is_recent_release (item))
 		gs_app_add_kudo (app, GS_APP_KUDO_RECENT_RELEASE);
@@ -873,12 +777,7 @@ gs_appstream_store_search (GsPlugin *plugin,
 {
 	GPtrArray *array;
 	gboolean ret = TRUE;
-	g_autoptr(AsProfileTask) ptask = NULL;
 
-	/* search categories for the search term */
-	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
-					  "appstream::search");
-	g_assert (ptask != NULL);
 	array = as_store_get_apps (store);
 	for (guint i = 0; i < array->len; i++) {
 		AsApp *item = g_ptr_array_index (array, i);
@@ -950,12 +849,8 @@ gs_appstream_store_add_category_apps (GsPlugin *plugin,
 {
 	GPtrArray *array;
 	GPtrArray *desktop_groups;
-	g_autoptr(AsProfileTask) ptask = NULL;
 
 	/* just look at each app in turn */
-	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
-					  "appstream::add-category-apps");
-	g_assert (ptask != NULL);
 	array = as_store_get_apps (store);
 	desktop_groups = gs_category_get_desktop_groups (category);
 	if (desktop_groups->len == 0) {
@@ -998,12 +893,8 @@ gs_appstream_store_add_categories (GsPlugin *plugin,
 				   GError **error)
 {
 	GPtrArray *array;
-	g_autoptr(AsProfileTask) ptask = NULL;
 
 	/* find out how many packages are in each category */
-	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
-					  "appstream::add-categories");
-	g_assert (ptask != NULL);
 	array = as_store_get_apps (store);
 	for (guint i = 0; i < array->len; i++) {
 		AsApp *app = g_ptr_array_index (array, i);
@@ -1026,14 +917,7 @@ gs_appstream_add_popular (GsPlugin *plugin,
 			  GCancellable *cancellable,
 			  GError **error)
 {
-	GPtrArray *array;
-	g_autoptr(AsProfileTask) ptask = NULL;
-
-	/* find out how many packages are in each category */
-	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
-					  "appstream::add-popular");
-	g_assert (ptask != NULL);
-	array = as_store_get_apps (store);
+	GPtrArray *array = as_store_get_apps (store);
 	for (guint i = 0; i < array->len; i++) {
 		g_autoptr(GsApp) app = NULL;
 		AsApp *item = g_ptr_array_index (array, i);
@@ -1073,14 +957,7 @@ gs_appstream_add_recent (GsPlugin *plugin,
 			 GCancellable *cancellable,
 			 GError **error)
 {
-	GPtrArray *array;
-	g_autoptr(AsProfileTask) ptask = NULL;
-
-	/* find out how many packages are in each category */
-	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
-					  "appstream::add-recent");
-	g_assert (ptask != NULL);
-	array = as_store_get_apps (store);
+	GPtrArray *array = as_store_get_apps (store);
 	for (guint i = 0; i < array->len; i++) {
 		g_autoptr(GsApp) app = NULL;
 		AsApp *item = g_ptr_array_index (array, i);
@@ -1096,6 +973,107 @@ gs_appstream_add_recent (GsPlugin *plugin,
 	return TRUE;
 }
 
+static void
+_g_ptr_array_add_str_uniq (GPtrArray *array, const gchar *str)
+{
+	if (str == NULL)
+		return;
+	for (guint i = 0; i < array->len; i++) {
+		const gchar *str_tmp = g_ptr_array_index (array, i);
+		if (g_strcmp0 (str, str_tmp) == 0)
+			return;
+	}
+	g_ptr_array_add (array, g_strdup (str));
+}
+
+/* add the component ID for any matching <provide><id/></provide> value */
+static void
+gs_appstream_add_alternates_new_id (GPtrArray *array, AsApp *item, const gchar *id)
+{
+	GPtrArray *provides = as_app_get_provides (item);
+	for (guint i = 0; i < provides->len; i++) {
+		AsProvide *provide = g_ptr_array_index (provides, i);
+		if (as_provide_get_kind (provide) == AS_PROVIDE_KIND_ID &&
+		    g_strcmp0 (as_provide_get_value (provide), id) == 0) {
+			_g_ptr_array_add_str_uniq (array, as_app_get_id (item));
+			break;
+		}
+	}
+
+}
+
+/* add all <provide><id/></provide> values for a matching component ID */
+static void
+gs_appstream_add_alternates_old_id (GPtrArray *array, AsApp *item, const gchar *id)
+{
+	GPtrArray *provides;
+	if (g_strcmp0 (as_app_get_id (item), id) != 0)
+		return;
+	provides = as_app_get_provides (item);
+	for (guint i = 0; i < provides->len; i++) {
+		AsProvide *provide = g_ptr_array_index (provides, i);
+		if (as_provide_get_kind (provide) == AS_PROVIDE_KIND_ID)
+			_g_ptr_array_add_str_uniq (array, as_provide_get_value (provide));
+	}
+}
+
+/* find any matching package names */
+static void
+gs_appstream_add_alternates_source (GPtrArray *array, AsApp *item, const gchar *source)
+{
+	GPtrArray *item_pkgnames = as_app_get_pkgnames (item);
+	for (guint i = 0; i < item_pkgnames->len; i++) {
+		const gchar *pkgname = g_ptr_array_index (item_pkgnames, i);
+		if (g_strcmp0 (pkgname, source) == 0) {
+			_g_ptr_array_add_str_uniq (array, as_app_get_id (item));
+			break;
+		}
+	}
+}
+
+gboolean
+gs_appstream_add_alternates (GsPlugin *plugin,
+			     AsStore *store,
+			     GsApp *app,
+			     GsAppList *list,
+			     GCancellable *cancellable,
+			     GError **error)
+{
+	GPtrArray *apps = as_store_get_apps (store);
+	GPtrArray *ids = g_ptr_array_new_with_free_func (g_free);
+
+	/* find apps that provide the new name */
+	for (guint i = 0; i < apps->len; i++) {
+		AsApp *item = g_ptr_array_index (apps, i);
+		GPtrArray *sources = gs_app_get_sources (app);
+
+		/* actual ID */
+		if (g_strcmp0 (as_app_get_id (item), gs_app_get_id (app)) == 0)
+			_g_ptr_array_add_str_uniq (ids, as_app_get_id (item));
+
+		/* new ID -> old ID */
+		gs_appstream_add_alternates_old_id (ids, item, gs_app_get_id (app));
+
+		/* old ID -> new ID */
+		gs_appstream_add_alternates_new_id (ids, item, gs_app_get_id (app));
+
+		/* find apps that use the same pkgname */
+		for (guint j = 0; j < sources->len; j++) {
+			const gchar *source = g_ptr_array_index (sources, j);
+			gs_appstream_add_alternates_source (ids, item, source);
+		}
+	}
+
+	/* add all results */
+	for (guint i = 0; i < ids->len; i++) {
+		const gchar *id = g_ptr_array_index (ids, i);
+		g_autoptr(GsApp) app2 = gs_app_new (id);
+		gs_app_add_quirk (app2, AS_APP_QUIRK_MATCH_ANY_PREFIX);
+		gs_app_list_add (list, app2);
+	}
+	return TRUE;
+}
+
 gboolean
 gs_appstream_add_featured (GsPlugin *plugin,
 			   AsStore *store,
@@ -1103,14 +1081,7 @@ gs_appstream_add_featured (GsPlugin *plugin,
 			   GCancellable *cancellable,
 			   GError **error)
 {
-	GPtrArray *array;
-	g_autoptr(AsProfileTask) ptask = NULL;
-
-	/* find out how many packages are in each category */
-	ptask = as_profile_start_literal (gs_plugin_get_profile (plugin),
-					  "appstream::add-featured");
-	g_assert (ptask != NULL);
-	array = as_store_get_apps (store);
+	GPtrArray *array = as_store_get_apps (store);
 	for (guint i = 0; i < array->len; i++) {
 		g_autoptr(GsApp) app = NULL;
 		AsApp *item = g_ptr_array_index (array, i);
