@@ -98,8 +98,21 @@ gs_flatpak_claim_app_list (GsFlatpak *self, GsAppList *list)
 }
 
 static void
-gs_flatpak_set_kind_from_flatpak (GsApp *app, FlatpakRef *xref)
+gs_flatpak_set_metadata (GsFlatpak *self, GsApp *app, FlatpakRef *xref)
 {
+	/* core */
+	gs_flatpak_claim_app (self, app);
+	gs_app_set_branch (app, flatpak_ref_get_branch (xref));
+	gs_plugin_refine_item_scope (self, app);
+
+	/* flatpak specific */
+	gs_flatpak_app_set_ref_kind (app, flatpak_ref_get_kind (xref));
+	gs_flatpak_app_set_ref_name (app, flatpak_ref_get_name (xref));
+	gs_flatpak_app_set_ref_arch (app, flatpak_ref_get_arch (xref));
+	gs_flatpak_app_set_ref_branch (app, flatpak_ref_get_branch (xref));
+	gs_flatpak_app_set_commit (app, flatpak_ref_get_commit (xref));
+
+	/* map the flatpak kind to the gnome-software kind */
 	if (flatpak_ref_get_kind (xref) == FLATPAK_REF_KIND_APP) {
 		gs_app_set_kind (app, AS_APP_KIND_DESKTOP);
 	} else if (flatpak_ref_get_kind (xref) == FLATPAK_REF_KIND_RUNTIME) {
@@ -119,28 +132,6 @@ gs_flatpak_set_kind_from_flatpak (GsApp *app, FlatpakRef *xref)
 	}
 }
 
-static void
-gs_flatpak_set_metadata (GsFlatpak *self, GsApp *app, FlatpakRef *xref)
-{
-	/* core */
-	gs_flatpak_claim_app (self, app);
-	gs_app_set_branch (app, flatpak_ref_get_branch (xref));
-	gs_plugin_refine_item_scope (self, app);
-
-	/* flatpak specific */
-	gs_flatpak_app_set_ref_kind (app, flatpak_ref_get_kind (xref));
-	gs_flatpak_app_set_ref_name (app, flatpak_ref_get_name (xref));
-	gs_flatpak_app_set_ref_arch (app, flatpak_ref_get_arch (xref));
-	gs_flatpak_app_set_ref_branch (app, flatpak_ref_get_branch (xref));
-	gs_flatpak_app_set_commit (app, flatpak_ref_get_commit (xref));
-
-	/* map the flatpak kind to the gnome-software kind */
-	if (gs_app_get_kind (app) == AS_APP_KIND_UNKNOWN ||
-	    gs_app_get_kind (app) == AS_APP_KIND_GENERIC) {
-		gs_flatpak_set_kind_from_flatpak (app, xref);
-	}
-}
-
 static GsApp *
 gs_flatpak_create_app (GsFlatpak *self, FlatpakRef *xref)
 {
@@ -244,52 +235,22 @@ gs_flatpak_remove_prefixed_names (AsApp *app)
 	}
 }
 
-#if !FLATPAK_CHECK_VERSION(1,1,1)
-static gchar *
-gs_flatpak_get_xremote_main_ref (GsFlatpak *self, FlatpakRemote *xremote, GError **error)
-{
-	g_autoptr(GFile) dir = NULL;
-	g_autofree gchar *dir_path = NULL;
-	g_autofree gchar *config_fn = NULL;
-	g_autofree gchar *group = NULL;
-	g_autofree gchar *main_ref = NULL;
-	g_autoptr(GKeyFile) kf = NULL;
-
-	/* figure out the path to the config keyfile */
-	dir = flatpak_installation_get_path (self->installation);
-	if (dir == NULL)
-		return NULL;
-	dir_path = g_file_get_path (dir);
-	if (dir_path == NULL)
-		return NULL;
-	config_fn = g_build_filename (dir_path, "repo", "config", NULL);
-
-	kf = g_key_file_new ();
-	if (!g_key_file_load_from_file (kf, config_fn, G_KEY_FILE_NONE, error))
-		return NULL;
-
-	group = g_strdup_printf ("remote \"%s\"", flatpak_remote_get_name (xremote));
-	main_ref = g_key_file_get_string (kf, group, "xa.main-ref", error);
-	return g_steal_pointer (&main_ref);
-}
-#endif
-
 static gboolean
 gs_flatpak_add_apps_from_xremote (GsFlatpak *self,
 				  FlatpakRemote *xremote,
 				  GCancellable *cancellable,
 				  GError **error)
 {
+	GPtrArray *apps;
 	g_autofree gchar *appstream_dir_fn = NULL;
 	g_autofree gchar *appstream_fn = NULL;
 	g_autofree gchar *default_branch = NULL;
-	g_autofree gchar *main_ref = NULL;
+	g_autofree gchar *only_app_id = NULL;
 	g_autoptr(AsStore) store = NULL;
 	g_autoptr(GFile) appstream_dir = NULL;
 	g_autoptr(GFile) file = NULL;
 	g_autoptr(GSettings) settings = NULL;
 	g_autoptr(GPtrArray) app_filtered = NULL;
-	g_autoptr(GPtrArray) apps = NULL;
 
 	/* get the AppStream data location */
 	appstream_dir = flatpak_remote_get_appstream_dir (xremote, NULL);
@@ -328,11 +289,7 @@ gs_flatpak_add_apps_from_xremote (GsFlatpak *self,
 	}
 
 	/* override the *AppStream* origin */
-#if AS_CHECK_VERSION(0,7,15)
-	apps = as_store_dup_apps (store);
-#else
-	apps = g_ptr_array_ref (as_store_get_apps (store));
-#endif
+	apps = as_store_get_apps (store);
 	for (guint i = 0; i < apps->len; i++) {
 		AsApp *app = g_ptr_array_index (apps, i);
 		as_app_set_origin (app, flatpak_remote_get_name (xremote));
@@ -340,14 +297,10 @@ gs_flatpak_add_apps_from_xremote (GsFlatpak *self,
 
 	/* only add the specific app for noenumerate=true */
 	if (flatpak_remote_get_noenumerate (xremote)) {
-#if FLATPAK_CHECK_VERSION(1,1,1)
-		main_ref = flatpak_remote_get_main_ref (xremote);
-#else
-		g_autoptr(GError) error_local = NULL;
-		main_ref = gs_flatpak_get_xremote_main_ref (self, xremote, &error_local);
-		if (main_ref == NULL)
-			g_warning ("failed to get main ref: %s", error_local->message);
-#endif
+		g_autofree gchar *tmp = NULL;
+		tmp = g_strdup (flatpak_remote_get_name (xremote));
+		g_strdelimit (tmp, "-", '\0');
+		only_app_id = g_strdup_printf ("%s.desktop", tmp);
 	}
 
 	/* do we want to filter to the default branch */
@@ -361,12 +314,10 @@ gs_flatpak_add_apps_from_xremote (GsFlatpak *self,
 		AsApp *app = g_ptr_array_index (apps, i);
 
 		/* filter to app */
-		if (flatpak_remote_get_noenumerate (xremote)) {
-			AsBundle *bundle = as_app_get_bundle_default (app);
-			if (bundle == NULL || main_ref == NULL)
-				continue;
-			if (g_strcmp0 (as_bundle_get_id (bundle), main_ref) != 0)
-				continue;
+		if (only_app_id != NULL &&
+		    g_strcmp0 (as_app_get_id (app), only_app_id) != 0) {
+			as_app_set_kind (app, AS_APP_KIND_UNKNOWN);
+			continue;
 		}
 
 		/* filter by branch */
@@ -397,6 +348,19 @@ gs_flatpak_add_apps_from_xremote (GsFlatpak *self,
 	return TRUE;
 }
 
+static gchar *
+gs_flatpak_discard_desktop_suffix (const gchar *app_id)
+{
+	const gchar *desktop_suffix = ".desktop";
+	guint app_prefix_len;
+
+	if (!g_str_has_suffix (app_id, desktop_suffix))
+		return g_strdup (app_id);
+
+	app_prefix_len = strlen (app_id) - strlen (desktop_suffix);
+	return g_strndup (app_id, app_prefix_len);
+}
+
 static void
 gs_flatpak_rescan_installed (GsFlatpak *self,
 			     GCancellable *cancellable,
@@ -424,6 +388,7 @@ gs_flatpak_rescan_installed (GsFlatpak *self,
 		g_autoptr(AsApp) app = NULL;
 		g_autoptr(AsFormat) format = as_format_new ();
 		g_autoptr(FlatpakInstalledRef) app_ref = NULL;
+		g_autofree gchar *app_id = NULL;
 
 		/* ignore */
 		if (g_strcmp0 (fn, "mimeinfo.cache") == 0)
@@ -458,25 +423,17 @@ gs_flatpak_rescan_installed (GsFlatpak *self,
 		as_format_set_filename (format, fn_desktop);
 		as_app_add_format (app, format);
 
+		app_id = gs_flatpak_discard_desktop_suffix (fn);
 		app_ref = flatpak_installation_get_current_installed_app (self->installation,
-									  as_app_get_id (app),
+									  app_id,
 									  cancellable,
 									  &error_local);
 		if (app_ref == NULL) {
 			g_warning ("Could not get app (from ID '%s') for installed desktop "
-				   "file %s: %s", as_app_get_id (app), fn_desktop, error_local->message);
+				   "file %s: %s", app_id, fn_desktop, error_local->message);
 			continue;
 		}
 
-		/* add the bundle info */
-		if (as_app_get_bundle_default (app) == NULL) {
-			g_autoptr(AsBundle) bundle = as_bundle_new ();
-			g_autofree gchar *ref = flatpak_ref_format_ref (FLATPAK_REF (app_ref));
-			as_bundle_set_kind (bundle, AS_BUNDLE_KIND_FLATPAK);
-			as_bundle_set_id (bundle, ref);
-			as_app_add_bundle (app, bundle);
-		}
-
 		as_app_set_branch (app, flatpak_ref_get_branch (FLATPAK_REF (app_ref)));
 		as_app_set_icon_path (app, path_exports);
 		as_app_add_keyword (app, NULL, "flatpak");
@@ -1928,9 +1885,6 @@ gs_flatpak_refine_app (GsFlatpak *self,
 	if (!gs_flatpak_refine_appstream (self, app, error))
 		return FALSE;
 
-	/* flatpak apps can always be removed */
-	gs_app_remove_quirk (app, AS_APP_QUIRK_COMPULSORY);
-
 	/* AppStream sets the source to appname/arch/branch */
 	if (!gs_refine_item_metadata (self, app, cancellable, error)) {
 		g_prefix_error (error, "failed to get metadata: ");
@@ -2060,7 +2014,7 @@ gs_flatpak_launch (GsFlatpak *self,
 					     GS_PLUGIN_ERROR,
 					     GS_PLUGIN_ERROR_NOT_SUPPORTED,
 					     "runtime is not installed");
-			gs_utils_error_add_origin_id (error, runtime);
+			gs_utils_error_add_unique_id (error, runtime);
 			gs_plugin_cache_add (self->plugin, NULL, runtime);
 			return FALSE;
 		}
@@ -2498,6 +2452,21 @@ gs_flatpak_add_featured (GsFlatpak *self,
 	return TRUE;
 }
 
+gboolean
+gs_flatpak_add_alternates (GsFlatpak *self,
+			   GsApp *app,
+			   GsAppList *list,
+			   GCancellable *cancellable,
+			   GError **error)
+{
+	g_autoptr(GsAppList) list_tmp = gs_app_list_new ();
+	if (!gs_appstream_add_alternates (self->plugin, self->store, app, list_tmp,
+					  cancellable, error))
+		return FALSE;
+	gs_app_list_add_list (list, list_tmp);
+	return TRUE;
+}
+
 gboolean
 gs_flatpak_add_recent (GsFlatpak *self,
 		       GsAppList *list,
