@@ -521,12 +521,9 @@ gs_plugin_app_remove (GsPlugin *plugin,
 		gs_flatpak_error_convert (error);
 		return FALSE;
 	}
-
-	/* run transaction */
-	gs_app_set_state (app, AS_APP_STATE_REMOVING);
 	if (!gs_flatpak_transaction_run (transaction, cancellable, error)) {
+		g_prefix_error (error, "failed to run transaction for %s: ", ref);
 		gs_flatpak_error_convert (error);
-		gs_app_set_state_recover (app);
 		return FALSE;
 	}
 
@@ -660,10 +657,10 @@ gs_plugin_app_install (GsPlugin *plugin,
 	}
 
 	/* run transaction */
-	gs_app_set_state (app, AS_APP_STATE_INSTALLING);
 	if (!gs_flatpak_transaction_run (transaction, cancellable, error)) {
+		g_prefix_error (error, "failed to run transaction for %s: ",
+				gs_app_get_unique_id (app));
 		gs_flatpak_error_convert (error);
-		gs_app_set_state_recover (app);
 		return FALSE;
 	}
 
@@ -684,22 +681,17 @@ gs_plugin_app_install (GsPlugin *plugin,
 }
 
 gboolean
-gs_plugin_update (GsPlugin *plugin,
-                  GsAppList *list,
-                  GCancellable *cancellable,
-                  GError **error)
+gs_plugin_update_app (GsPlugin *plugin,
+		      GsApp *app,
+		      GCancellable *cancellable,
+		      GError **error)
 {
 	GsFlatpak *flatpak;
 	g_autoptr(FlatpakTransaction) transaction = NULL;
-	g_autoptr(GsAppList) list_tmp = gs_app_list_new ();
+	g_autofree gchar *ref = NULL;
 
 	/* not supported */
-	for (guint i = 0; i < gs_app_list_length (list); i++) {
-		GsApp *app = gs_app_list_index (list, i);
-		flatpak = gs_plugin_flatpak_get_handler (plugin, app);
-		if (flatpak != NULL)
-			gs_app_list_add (list_tmp, app);
-	}
+	flatpak = gs_plugin_flatpak_get_handler (plugin, app);
 	if (flatpak == NULL)
 		return TRUE;
 
@@ -709,29 +701,14 @@ gs_plugin_update (GsPlugin *plugin,
 		gs_flatpak_error_convert (error);
 		return FALSE;
 	}
-
-	for (guint i = 0; i < gs_app_list_length (list_tmp); i++) {
-		GsApp *app = gs_app_list_index (list_tmp, i);
-		g_autofree gchar *ref = NULL;
-
-		ref = gs_flatpak_app_get_ref_display (app);
-		if (!flatpak_transaction_add_update (transaction, ref, NULL, NULL, error)) {
-			g_prefix_error (error, "failed to add update ref %s: ", ref);
-			gs_flatpak_error_convert (error);
-			return FALSE;
-		}
-	}
-
-	/* run transaction */
-	for (guint i = 0; i < gs_app_list_length (list_tmp); i++) {
-		GsApp *app = gs_app_list_index (list_tmp, i);
-		gs_app_set_state (app, AS_APP_STATE_INSTALLING);
+	ref = gs_flatpak_app_get_ref_display (app);
+	if (!flatpak_transaction_add_update (transaction, ref, NULL, NULL, error)) {
+		g_prefix_error (error, "failed to add update ref %s: ", ref);
+		gs_flatpak_error_convert (error);
+		return FALSE;
 	}
 	if (!gs_flatpak_transaction_run (transaction, cancellable, error)) {
-		for (guint i = 0; i < gs_app_list_length (list_tmp); i++) {
-			GsApp *app = gs_app_list_index (list_tmp, i);
-			gs_app_set_state_recover (app);
-		}
+		g_prefix_error (error, "failed to run transaction for %s: ", ref);
 		gs_flatpak_error_convert (error);
 		return FALSE;
 	}
@@ -742,18 +719,12 @@ gs_plugin_update (GsPlugin *plugin,
 		gs_flatpak_error_convert (error);
 		return FALSE;
 	}
-	for (guint i = 0; i < gs_app_list_length (list_tmp); i++) {
-		GsApp *app = gs_app_list_index (list_tmp, i);
-		g_autofree gchar *ref = NULL;
-
-		ref = gs_flatpak_app_get_ref_display (app);
-		if (!gs_flatpak_refine_app (flatpak, app,
-					    GS_PLUGIN_REFINE_FLAGS_REQUIRE_RUNTIME,
-					    cancellable, error)) {
-			g_prefix_error (error, "failed to run refine for %s: ", ref);
-			gs_flatpak_error_convert (error);
-			return FALSE;
-		}
+	if (!gs_flatpak_refine_app (flatpak, app,
+				    GS_PLUGIN_REFINE_FLAGS_REQUIRE_RUNTIME,
+				    cancellable, error)) {
+		g_prefix_error (error, "failed to run refine for %s: ", ref);
+		gs_flatpak_error_convert (error);
+		return FALSE;
 	}
 	return TRUE;
 }
@@ -880,7 +851,7 @@ gs_plugin_flatpak_file_to_app_ref (GsPlugin *plugin,
 	ref = gs_flatpak_app_get_ref_display (app);
 	app_tmp = gs_plugin_flatpak_find_app_by_ref (plugin, ref, cancellable, NULL);
 	if (app_tmp != NULL)
-		return g_steal_pointer (&app_tmp);
+		return g_steal_pointer (&app);
 
 	/* force this to be 'any' scope for installation */
 	gs_app_set_scope (app, AS_APP_SCOPE_UNKNOWN);
@@ -1020,6 +991,22 @@ gs_plugin_add_popular (GsPlugin *plugin,
 	return TRUE;
 }
 
+gboolean
+gs_plugin_add_alternates (GsPlugin *plugin,
+			  GsApp *app,
+			  GsAppList *list,
+			  GCancellable *cancellable,
+			  GError **error)
+{
+	GsPluginData *priv = gs_plugin_get_data (plugin);
+	for (guint i = 0; i < priv->flatpaks->len; i++) {
+		GsFlatpak *flatpak = g_ptr_array_index (priv->flatpaks, i);
+		if (!gs_flatpak_add_alternates (flatpak, app, list, cancellable, error))
+			return FALSE;
+	}
+	return TRUE;
+}
+
 gboolean
 gs_plugin_add_featured (GsPlugin *plugin,
 			GsAppList *list,
