@@ -282,6 +282,7 @@ gs_plugin_fwupd_new_app_from_device (GsPlugin *plugin, FwupdDevice *dev)
 
 	/* default stuff */
 	gs_app_set_kind (app, AS_APP_KIND_FIRMWARE);
+	gs_app_set_bundle_kind (app, AS_BUNDLE_KIND_CABINET);
 	gs_app_add_quirk (app, AS_APP_QUIRK_NOT_LAUNCHABLE);
 	gs_app_set_management_plugin (app, "fwupd");
 	gs_app_add_category (app, "System");
@@ -672,7 +673,7 @@ gs_plugin_fwupd_refresh_remote (GsPlugin *plugin,
 				    _("Downloading firmware update signature…"));
 	data = gs_plugin_download_data (plugin, app_dl, url_sig, cancellable, error);
 	if (data == NULL) {
-		gs_utils_error_add_origin_id (error, priv->cached_origin);
+		gs_utils_error_add_unique_id (error, priv->cached_origin);
 		return FALSE;
 	}
 
@@ -714,7 +715,7 @@ gs_plugin_fwupd_refresh_remote (GsPlugin *plugin,
 	url = fwupd_remote_get_metadata_uri (remote);
 	if (!gs_plugin_download_file (plugin, app_dl, url, filename,
 				      cancellable, error)) {
-		gs_utils_error_add_origin_id (error, priv->cached_origin);
+		gs_utils_error_add_unique_id (error, priv->cached_origin);
 		return FALSE;
 	}
 
