@@ -26,6 +26,8 @@
 
 #include <gnome-software.h>
 
+#include "packagekit-common.h"
+
 /*
  * SECTION:
  * This adds historical updates to the application history.
@@ -155,6 +157,7 @@ gs_plugin_add_updates_historical (GsPlugin *plugin,
 		gs_app_add_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX);
 		gs_app_set_state (app, AS_APP_STATE_UNKNOWN);
 		gs_app_set_kind (app, AS_APP_KIND_OS_UPGRADE);
+		gs_app_set_bundle_kind (app, AS_BUNDLE_KIND_PACKAGE);
 		gs_app_set_install_date (app, mtime);
 		gs_app_set_metadata (app, "GnomeSoftware::Creator",
 				     gs_plugin_get_name (plugin));
@@ -175,12 +178,14 @@ gs_plugin_add_updates_historical (GsPlugin *plugin,
 		app = gs_app_new (NULL);
 		package_id = pk_package_get_id (pkg);
 		split = g_strsplit (package_id, ";", 4);
+		gs_plugin_packagekit_set_packaging_format (plugin, app);
 		gs_app_add_source (app, split[0]);
 		gs_app_set_update_version (app, split[1]);
 		gs_app_set_management_plugin (app, "packagekit");
 		gs_app_add_source_id (app, package_id);
 		gs_app_set_state (app, AS_APP_STATE_UPDATABLE);
 		gs_app_set_kind (app, AS_APP_KIND_GENERIC);
+		gs_app_set_bundle_kind (app, AS_BUNDLE_KIND_PACKAGE);
 		gs_app_set_install_date (app, mtime);
 		gs_app_set_metadata (app, "GnomeSoftware::Creator",
 				     gs_plugin_get_name (plugin));
