@@ -44,17 +44,11 @@ struct GsPluginData {
 };
 
 static void
-gs_plugin_packagekit_updates_changed_cb (PkControl *control, GsPlugin *plugin)
+gs_plugin_packagekit_cache_invalid_cb (PkControl *control, GsPlugin *plugin)
 {
 	gs_plugin_updates_changed (plugin);
 }
 
-static void
-gs_plugin_packagekit_repo_list_changed_cb (PkControl *control, GsPlugin *plugin)
-{
-	gs_plugin_reload (plugin);
-}
-
 void
 gs_plugin_initialize (GsPlugin *plugin)
 {
@@ -62,9 +56,9 @@ gs_plugin_initialize (GsPlugin *plugin)
 	priv->client = pk_client_new ();
 	priv->control = pk_control_new ();
 	g_signal_connect (priv->control, "updates-changed",
-			  G_CALLBACK (gs_plugin_packagekit_updates_changed_cb), plugin);
+			  G_CALLBACK (gs_plugin_packagekit_cache_invalid_cb), plugin);
 	g_signal_connect (priv->control, "repo-list-changed",
-			  G_CALLBACK (gs_plugin_packagekit_repo_list_changed_cb), plugin);
+			  G_CALLBACK (gs_plugin_packagekit_cache_invalid_cb), plugin);
 	pk_client_set_background (priv->client, FALSE);
 	pk_client_set_cache_age (priv->client, G_MAXUINT);
 
@@ -87,6 +81,7 @@ gs_plugin_adopt_app (GsPlugin *plugin, GsApp *app)
 	if (gs_app_get_bundle_kind (app) == AS_BUNDLE_KIND_PACKAGE &&
 	    gs_app_get_scope (app) == AS_APP_SCOPE_SYSTEM) {
 		gs_app_set_management_plugin (app, "packagekit");
+		gs_plugin_packagekit_set_packaging_format (plugin, app);
 		return;
 	}
 }
@@ -166,7 +161,7 @@ gs_plugin_packagekit_resolve_packages (GsPlugin *plugin,
                                        GError **error)
 {
 	PkBitfield filter;
-	g_autoptr(GsAppList) resolve2_list = NULL;
+	GsAppList *resolve2_list;
 
 	/* first, try to resolve packages with ARCH filter */
 	filter = pk_bitfield_from_enums (PK_FILTER_ENUM_NEWEST,
