@@ -157,6 +157,7 @@ gs_plugin_add_sources (GsPlugin *plugin,
 		app = gs_app_new (id);
 		gs_app_set_management_plugin (app, gs_plugin_get_name (plugin));
 		gs_app_set_kind (app, AS_APP_KIND_SOURCE);
+		gs_app_set_bundle_kind (app, AS_BUNDLE_KIND_PACKAGE);
 		gs_app_add_quirk (app, AS_APP_QUIRK_NOT_LAUNCHABLE);
 		gs_app_set_state (app, pk_repo_detail_get_enabled (rd) ?
 				  AS_APP_STATE_INSTALLED : AS_APP_STATE_AVAILABLE);
@@ -196,7 +197,7 @@ gs_plugin_app_origin_repo_enable (GsPlugin *plugin,
 	                                 gs_packagekit_helper_cb, helper,
 	                                 error);
 	if (!gs_plugin_packagekit_results_valid (results, error)) {
-		gs_utils_error_add_origin_id (error, app);
+		gs_utils_error_add_unique_id (error, app);
 		return FALSE;
 	}
 
@@ -229,7 +230,7 @@ gs_plugin_repo_enable (GsPlugin *plugin,
 					 error);
 	if (!gs_plugin_packagekit_results_valid (results, error)) {
 		gs_app_set_state_recover (app);
-		gs_utils_error_add_origin_id (error, app);
+		gs_utils_error_add_unique_id (error, app);
 		return FALSE;
 	}
 
@@ -452,7 +453,7 @@ gs_plugin_repo_disable (GsPlugin *plugin,
 					 error);
 	if (!gs_plugin_packagekit_results_valid (results, error)) {
 		gs_app_set_state_recover (app);
-		gs_utils_error_add_origin_id (error, app);
+		gs_utils_error_add_unique_id (error, app);
 		return FALSE;
 	}
 
@@ -540,6 +541,7 @@ gs_plugin_packagekit_build_update_app (GsPlugin *plugin, PkPackage *package)
 	if (app != NULL)
 		return app;
 	app = gs_app_new (NULL);
+	gs_plugin_packagekit_set_packaging_format (plugin, app);
 	gs_app_add_source (app, pk_package_get_name (package));
 	gs_app_add_source_id (app, pk_package_get_id (package));
 	gs_app_set_name (app, GS_APP_QUALITY_LOWEST,
