@@ -39,7 +39,6 @@ struct GsPluginData {
 	GPermission		*permission;
 	gboolean		 is_triggered;
 	GHashTable		*hash_prepared;
-	GMutex			 hash_prepared_mutex;
 };
 
 void
@@ -49,7 +48,6 @@ gs_plugin_initialize (GsPlugin *plugin)
 	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "packagekit-refresh");
 	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "packagekit-refine");
 	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_BEFORE, "generic-updates");
-	g_mutex_init (&priv->hash_prepared_mutex);
 	priv->hash_prepared = g_hash_table_new_full (g_str_hash, g_str_equal,
 						     g_free, NULL);
 }
@@ -59,7 +57,6 @@ gs_plugin_destroy (GsPlugin *plugin)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	g_hash_table_unref (priv->hash_prepared);
-	g_mutex_clear (&priv->hash_prepared_mutex);
 	if (priv->monitor != NULL)
 		g_object_unref (priv->monitor);
 	if (priv->monitor_trigger != NULL)
@@ -83,7 +80,6 @@ gs_plugin_systemd_update_cache (GsPlugin *plugin, GError **error)
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	g_autoptr(GError) error_local = NULL;
 	g_auto(GStrv) package_ids = NULL;
-	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&priv->hash_prepared_mutex);
 
 	/* invalidate */
 	g_hash_table_remove_all (priv->hash_prepared);
@@ -145,52 +141,30 @@ gs_plugin_systemd_trigger_changed_cb (GFileMonitor *monitor,
 	gs_plugin_systemd_updates_refresh_is_triggered (plugin, NULL);
 }
 
-static void
-gs_plugin_systemd_refine_app (GsPlugin *plugin, GsApp *app)
+gboolean
+gs_plugin_refine_app (GsPlugin *plugin,
+		      GsApp *app,
+		      GsPluginRefineFlags flags,
+		      GCancellable *cancellable,
+		      GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	const gchar *package_id;
-	g_autoptr(GMutexLocker) locker = NULL;
 
-	/* only process this app if was created by this plugin */
-	if (g_strcmp0 (gs_app_get_management_plugin (app), "packagekit") != 0)
-		return;
-
-	/* the package is already downloaded */
-	package_id = gs_app_get_source_id_default (app);
-	if (package_id == NULL)
-		return;
-	locker = g_mutex_locker_new (&priv->hash_prepared_mutex);
-	if (g_hash_table_lookup (priv->hash_prepared, package_id) != NULL)
-		gs_app_set_size_download (app, 0);
-}
-
-gboolean
-gs_plugin_refine (GsPlugin *plugin,
-                  GsAppList *list,
-                  GsPluginRefineFlags flags,
-                  GCancellable *cancellable,
-                  GError **error)
-{
 	/* not now */
 	if ((flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_SIZE) == 0)
 		return TRUE;
 
-	/* re-read /var/lib/PackageKit/prepared-update */
-	if (!gs_plugin_systemd_update_cache (plugin, error))
-		return FALSE;
+	/* only process this app if was created by this plugin */
+	if (g_strcmp0 (gs_app_get_management_plugin (app), "packagekit") != 0)
+		return TRUE;
 
-	for (guint i = 0; i < gs_app_list_length (list); i++) {
-		GsApp *app = gs_app_list_index (list, i);
-		GsAppList *related = gs_app_get_related (app);
-		/* refine the app itself */
-		gs_plugin_systemd_refine_app (plugin, app);
-		/* and anything related for proxy apps */
-		for (guint j = 0; j < gs_app_list_length (related); j++) {
-			GsApp *app_related = gs_app_list_index (related, j);
-			gs_plugin_systemd_refine_app (plugin, app_related);
-		}
-	}
+	/* the package is already downloaded */
+	package_id = gs_app_get_source_id_default (app);
+	if (package_id == NULL)
+		return TRUE;
+	if (g_hash_table_lookup (priv->hash_prepared, package_id) != NULL)
+		gs_app_set_size_download (app, 0);
 
 	return TRUE;
 }
@@ -284,12 +258,9 @@ gs_plugin_update (GsPlugin *plugin,
 		GsApp *app = gs_app_list_index (list, i);
 		GsAppList *related = gs_app_get_related (app);
 
-		/* try to trigger this app */
-		if (!gs_app_has_quirk (app, AS_APP_QUIRK_IS_PROXY)) {
-			if (!_systemd_trigger_app (plugin, app, cancellable, error))
-				return FALSE;
-			continue;
-		}
+		/* not a proxy, which is somewhat odd... */
+		if (!gs_app_has_quirk (app, AS_APP_QUIRK_IS_PROXY))
+			return _systemd_trigger_app (plugin, app, cancellable, error);
 
 		/* try to trigger each related app */
 		for (guint j = 0; j < gs_app_list_length (related); j++) {
