@@ -124,6 +124,9 @@ snapd_error_convert (GError **perror)
 		case SNAPD_ERROR_TWO_FACTOR_INVALID:
 			error->code = GS_PLUGIN_ERROR_AUTH_INVALID;
 			break;
+		case SNAPD_ERROR_AUTH_CANCELLED:
+			error->code = GS_PLUGIN_ERROR_CANCELLED;
+			break; 
 		case SNAPD_ERROR_CONNECTION_FAILED:
 		case SNAPD_ERROR_WRITE_FAILED:
 		case SNAPD_ERROR_READ_FAILED:
@@ -303,6 +306,7 @@ snap_to_app (GsPlugin *plugin, SnapdSnap *snap)
 	if (app == NULL) {
 		app = gs_app_new (NULL);
 		gs_app_set_from_unique_id (app, unique_id);
+		gs_app_set_bundle_kind (app, AS_BUNDLE_KIND_SNAP);
 		gs_app_set_metadata (app, "snap::name", snapd_snap_get_name (snap));
 		gs_plugin_cache_add (plugin, unique_id, app);
 	}
@@ -820,6 +824,8 @@ gs_plugin_refine_app (GsPlugin *plugin,
 	if (developer_name == NULL)
 		developer_name = snapd_snap_get_publisher_username (snap);
 	gs_app_set_developer_name (app, developer_name);
+	if (snapd_snap_get_publisher_validation (snap) == SNAPD_PUBLISHER_VALIDATION_VERIFIED)
+		gs_app_add_quirk (app, AS_APP_QUIRK_DEVELOPER_VERIFIED);
 
 	snap = local_snap != NULL ? local_snap : store_snap;
 	gs_app_set_version (app, snapd_snap_get_version (snap));
