@@ -294,9 +294,6 @@ gs_extras_page_add_app (GsExtrasPage *self, GsApp *app, GsAppList *list, SearchD
 	app_row = gs_app_row_new (app);
 	gs_app_row_set_colorful (GS_APP_ROW (app_row), TRUE);
 	gs_app_row_set_show_buttons (GS_APP_ROW (app_row), TRUE);
-	if (!gs_app_has_quirk (app, AS_APP_QUIRK_PROVENANCE) ||
-	    gs_utils_list_has_app_fuzzy (list, app))
-		gs_app_row_set_show_source (GS_APP_ROW (app_row), TRUE);
 
 	g_object_set_data_full (G_OBJECT (app_row), "missing-title", g_strdup (search_data->title), g_free);
 
@@ -663,10 +660,8 @@ gs_extras_page_load (GsExtrasPage *self, GPtrArray *array_search_data)
 	guint i;
 
 	/* cancel any pending searches */
-	if (self->search_cancellable != NULL) {
-		g_cancellable_cancel (self->search_cancellable);
-		g_object_unref (self->search_cancellable);
-	}
+	g_cancellable_cancel (self->search_cancellable);
+	g_clear_object (&self->search_cancellable);
 	self->search_cancellable = g_cancellable_new ();
 
 	if (array_search_data != NULL) {
@@ -692,7 +687,7 @@ gs_extras_page_load (GsExtrasPage *self, GPtrArray *array_search_data)
 			g_autoptr(GsPluginJob) plugin_job = NULL;
 			plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_SEARCH_FILES,
 							 "search", search_data->search_filename,
-									 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
+							 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 									 GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING |
 									 GS_PLUGIN_REFINE_FLAGS_ALLOW_PACKAGES,
 							 NULL);
@@ -708,7 +703,7 @@ gs_extras_page_load (GsExtrasPage *self, GPtrArray *array_search_data)
 			file = g_file_new_for_path (search_data->package_filename);
 			plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_FILE_TO_APP,
 							 "file", file,
-									 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
+							 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 									 GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING |
 									 GS_PLUGIN_REFINE_FLAGS_ALLOW_PACKAGES,
 							 NULL);
@@ -722,9 +717,8 @@ gs_extras_page_load (GsExtrasPage *self, GPtrArray *array_search_data)
 			g_debug ("searching what provides: '%s'", search_data->search);
 			plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_SEARCH_PROVIDES,
 							 "search", search_data->search,
-									 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
+							 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 									 GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION |
-									 GS_PLUGIN_REFINE_FLAGS_REQUIRE_PROVENANCE |
 									 GS_PLUGIN_REFINE_FLAGS_REQUIRE_HISTORY |
 									 GS_PLUGIN_REFINE_FLAGS_REQUIRE_SETUP_ACTION |
 									 GS_PLUGIN_REFINE_FLAGS_REQUIRE_DESCRIPTION |
@@ -1159,10 +1153,8 @@ gs_extras_page_dispose (GObject *object)
 {
 	GsExtrasPage *self = GS_EXTRAS_PAGE (object);
 
-	if (self->search_cancellable != NULL) {
-		g_cancellable_cancel (self->search_cancellable);
-		g_clear_object (&self->search_cancellable);
-	}
+	g_cancellable_cancel (self->search_cancellable);
+	g_clear_object (&self->search_cancellable);
 
 	g_clear_object (&self->sizegroup_image);
 	g_clear_object (&self->sizegroup_name);
