@@ -188,8 +188,7 @@ gs_installed_page_add_app (GsInstalledPage *self, GsAppList *list, GsApp *app)
 	app_row = gs_app_row_new (app);
 	gs_app_row_set_show_folders (GS_APP_ROW (app_row), TRUE);
 	gs_app_row_set_show_buttons (GS_APP_ROW (app_row), TRUE);
-	if (!gs_app_has_quirk (app, AS_APP_QUIRK_PROVENANCE) ||
-	    gs_utils_list_has_app_fuzzy (list, app))
+	if (gs_utils_list_has_app_fuzzy (list, app))
 		gs_app_row_set_show_source (GS_APP_ROW (app_row), TRUE);
 	g_signal_connect (app_row, "button-clicked",
 			  G_CALLBACK (gs_installed_page_app_remove_cb), self);
@@ -272,6 +271,7 @@ gs_installed_page_load (GsInstalledPage *self)
 		GS_PLUGIN_REFINE_FLAGS_REQUIRE_PROVENANCE |
 		GS_PLUGIN_REFINE_FLAGS_REQUIRE_DESCRIPTION |
 		GS_PLUGIN_REFINE_FLAGS_REQUIRE_LICENSE |
+		GS_PLUGIN_REFINE_FLAGS_REQUIRE_CATEGORIES |
 		GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING;
 
 	if (should_show_installed_size (self))
@@ -280,6 +280,7 @@ gs_installed_page_load (GsInstalledPage *self)
 	/* get installed apps */
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_GET_INSTALLED,
 					 "refine-flags", flags,
+					 "dedupe-flags", GS_APP_LIST_FILTER_FLAG_NONE,
 					 NULL);
 	gs_plugin_loader_job_process_async (self->plugin_loader,
 					    plugin_job,
@@ -326,6 +327,8 @@ gs_installed_page_switch_to (GsPage *page, gboolean scroll_up)
 	set_selection_mode (self, FALSE);
 	widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "buttonbox_main"));
 	gtk_widget_show (widget);
+	widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "menu_button"));
+	gtk_widget_show (widget);
 
 	gs_shell_update_button_select_visibility (self);
 
@@ -611,6 +614,8 @@ set_selection_mode (GsInstalledPage *self, gboolean selection_mode)
 		gtk_widget_show (self->button_select);
 		widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "buttonbox_main"));
 		gtk_widget_hide (widget);
+		widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "menu_button"));
+		gtk_widget_hide (widget);
 		widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "header_selection_menu_button"));
 		gtk_widget_show (widget);
 		widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "header_selection_label"));
@@ -623,6 +628,8 @@ set_selection_mode (GsInstalledPage *self, gboolean selection_mode)
 		gtk_widget_show (self->button_select);
 		widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "buttonbox_main"));
 		gtk_widget_show (widget);
+		widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "menu_button"));
+		gtk_widget_show (widget);
 		widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "header_selection_menu_button"));
 		gtk_widget_hide (widget);
 	}
