@@ -38,8 +38,6 @@ struct _GsPopularTile
 	GtkWidget	*eventbox;
 	GtkWidget	*stack;
 	GtkWidget	*stars;
-	GtkWidget	*label_origin;
-	guint		 app_state_changed_id;
 };
 
 G_DEFINE_TYPE (GsPopularTile, gs_popular_tile, GS_TYPE_APP_TILE)
@@ -58,8 +56,6 @@ app_state_changed_idle (gpointer user_data)
 	gboolean installed;
 	g_autofree gchar *name = NULL;
 
-	tile->app_state_changed_id = 0;
-
 	accessible = gtk_widget_get_accessible (GTK_WIDGET (tile));
 
 	switch (gs_app_get_state (tile->app)) {
@@ -87,14 +83,14 @@ app_state_changed_idle (gpointer user_data)
 		atk_object_set_description (accessible, gs_app_get_summary (tile->app));
 	}
 
+	g_object_unref (tile);
 	return G_SOURCE_REMOVE;
 }
 
 static void
 app_state_changed (GsApp *app, GParamSpec *pspec, GsPopularTile *tile)
 {
-	g_clear_handle_id (&tile->app_state_changed_id, g_source_remove);
-	tile->app_state_changed_id = g_idle_add (app_state_changed_idle, tile);
+	g_idle_add (app_state_changed_idle, g_object_ref (tile));
 }
 
 static void
@@ -105,9 +101,8 @@ gs_popular_tile_set_app (GsAppTile *app_tile, GsApp *app)
 
 	g_return_if_fail (GS_IS_APP (app) || app == NULL);
 
-	if (tile->app != NULL)
+	if (tile->app)
 		g_signal_handlers_disconnect_by_func (tile->app, app_state_changed, tile);
-	g_clear_handle_id (&tile->app_state_changed_id, g_source_remove);
 
 	g_set_object (&tile->app, app);
 	if (!app)
@@ -150,9 +145,9 @@ gs_popular_tile_destroy (GtkWidget *widget)
 {
 	GsPopularTile *tile = GS_POPULAR_TILE (widget);
 
-	if (tile->app != NULL)
+	if (tile->app)
 		g_signal_handlers_disconnect_by_func (tile->app, app_state_changed, tile);
-	g_clear_handle_id (&tile->app_state_changed_id, g_source_remove);
+
 	g_clear_object (&tile->app);
 
 	GTK_WIDGET_CLASS (gs_popular_tile_parent_class)->destroy (widget);
@@ -184,7 +179,6 @@ gs_popular_tile_class_init (GsPopularTileClass *klass)
 	gtk_widget_class_bind_template_child (widget_class, GsPopularTile, eventbox);
 	gtk_widget_class_bind_template_child (widget_class, GsPopularTile, stack);
 	gtk_widget_class_bind_template_child (widget_class, GsPopularTile, stars);
-	gtk_widget_class_bind_template_child (widget_class, GsPopularTile, label_origin);
 }
 
 GtkWidget *
@@ -199,23 +193,4 @@ gs_popular_tile_new (GsApp *app)
 	return GTK_WIDGET (tile);
 }
 
-void
-gs_popular_tile_show_source (GsPopularTile *tile, gboolean show_source)
-{
-	if (show_source) {
-		const gchar *hostname = gs_app_get_origin_hostname (tile->app);
-		if (hostname != NULL) {
-			/* TRANSLATORS: this refers to where the app came from */
-			g_autofree gchar *source_text = g_strdup_printf (_("Source: %s"),
-									 hostname);
-			gtk_label_set_label (GTK_LABEL (tile->label_origin), source_text);
-		} else {
-			/* if the hostname is not valid then we hide the source */
-			show_source = FALSE;
-		}
-	}
-
-	gtk_widget_set_visible (tile->label_origin, show_source);
-}
-
 /* vim: set noexpandtab: */
