@@ -752,7 +752,8 @@ get_os_name (void)
 }
 
 static void
-reload_cb (GsPluginLoader *plugin_loader, GsReposDialog *dialog)
+updates_changed_cb (GsPluginLoader *plugin_loader,
+                    GsReposDialog *dialog)
 {
 	reload_sources (dialog);
 	reload_third_party_repo (dialog);
@@ -762,8 +763,8 @@ static void
 set_plugin_loader (GsReposDialog *dialog, GsPluginLoader *plugin_loader)
 {
 	dialog->plugin_loader = g_object_ref (plugin_loader);
-	g_signal_connect (dialog->plugin_loader, "reload",
-	                  G_CALLBACK (reload_cb), dialog);
+	g_signal_connect (dialog->plugin_loader, "updates-changed",
+	                  G_CALLBACK (updates_changed_cb), dialog);
 }
 
 static void
@@ -772,14 +773,12 @@ gs_repos_dialog_dispose (GObject *object)
 	GsReposDialog *dialog = GS_REPOS_DIALOG (object);
 
 	if (dialog->plugin_loader != NULL) {
-		g_signal_handlers_disconnect_by_func (dialog->plugin_loader, reload_cb, dialog);
+		g_signal_handlers_disconnect_by_func (dialog->plugin_loader, updates_changed_cb, dialog);
 		g_clear_object (&dialog->plugin_loader);
 	}
 
-	if (dialog->cancellable != NULL) {
-		g_cancellable_cancel (dialog->cancellable);
-		g_clear_object (&dialog->cancellable);
-	}
+	g_cancellable_cancel (dialog->cancellable);
+	g_clear_object (&dialog->cancellable);
 	g_clear_object (&dialog->settings);
 	g_clear_object (&dialog->third_party_repo);
 
