@@ -81,6 +81,7 @@ typedef struct
 	gchar			*events_info_uri;
 	gboolean		 in_mode_change;
 	GsPage			*page_last;
+	GSimpleActionGroup	*auth_actions;
 } GsShellPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GsShell, gs_shell, G_TYPE_OBJECT)
@@ -224,17 +225,6 @@ gs_shell_clean_back_entry_stack (GsShell *shell)
 	}
 }
 
-static void
-gs_shell_update_account_button_visibility (GsShell *shell)
-{
-	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
-	GPtrArray *auths = gs_plugin_loader_get_auths (priv->plugin_loader);
-	GtkWidget *account_button;
-
-	account_button = GTK_WIDGET (gtk_builder_get_object (priv->builder, "account_button"));
-	gtk_widget_set_visible (account_button, auths->len > 0);
-}
-
 void
 gs_shell_change_mode (GsShell *shell,
 		      GsShellMode mode,
@@ -260,8 +250,12 @@ gs_shell_change_mode (GsShell *shell,
 	gtk_widget_hide (widget);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "buttonbox_main"));
 	gtk_widget_hide (widget);
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "menu_button"));
+	gtk_widget_hide (widget);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "header_selection_menu_button"));
 	gtk_widget_hide (widget);
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "origin_box"));
+	gtk_widget_hide (widget);
 
 	priv->in_mode_change = TRUE;
 	/* only show the search button in overview and search pages */
@@ -370,8 +364,6 @@ gs_shell_change_mode (GsShell *shell,
 	widget = gs_page_get_header_end_widget (page);
 	gs_shell_set_header_end_widget (shell, widget);
 
-	gs_shell_update_account_button_visibility (shell);
-
 	/* destroy any existing modals */
 	if (priv->modal_dialogs != NULL) {
 		gsize i = 0;
@@ -546,24 +538,13 @@ gs_shell_back_button_cb (GtkWidget *widget, GsShell *shell)
 }
 
 static void
-gs_shell_reload_cb (GsPluginLoader *plugin_loader, GsShell *shell)
-{
-	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
-	g_autoptr(GList) keys = g_hash_table_get_keys (priv->pages);
-	for (GList *l = keys; l != NULL; l = l->next) {
-		GsPage *page = GS_PAGE (g_hash_table_lookup (priv->pages, l->data));
-		gs_page_reload (page);
-	}
-}
-
-static void
-initial_refresh_done (GsLoadingPage *loading_page, gpointer data)
+initial_overview_load_done (GsOverviewPage *overview_page, gpointer data)
 {
 	GsPage *page;
 	GsShell *shell = data;
 	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
 
-	g_signal_handlers_disconnect_by_func (loading_page, initial_refresh_done, data);
+	g_signal_handlers_disconnect_by_func (overview_page, initial_overview_load_done, data);
 
 	page = GS_PAGE (gtk_builder_get_object (priv->builder, "updates_page"));
 	gs_page_reload (page);
@@ -571,14 +552,6 @@ initial_refresh_done (GsLoadingPage *loading_page, gpointer data)
 	gs_page_reload (page);
 
 	g_signal_emit (shell, signals[SIGNAL_LOADED], 0);
-
-	/* go to OVERVIEW, unless the "loading" callbacks changed mode already */
-	if (priv->mode == GS_SHELL_MODE_LOADING)
-		gs_shell_change_mode (shell, GS_SHELL_MODE_OVERVIEW, NULL, TRUE);
-
-	/* now that we're finished with the loading page, connect the reload signal handler */
-	g_signal_connect (priv->plugin_loader, "reload",
-	                  G_CALLBACK (gs_shell_reload_cb), shell);
 }
 
 static gboolean
@@ -661,12 +634,19 @@ search_mode_enabled_cb (GtkSearchBar *search_bar, GParamSpec *pspec, GsShell *sh
 }
 
 static void
-gs_shell_signin_button_cb (GtkButton *button, GsShell *shell)
+signin_activated_cb (GSimpleAction *action, GVariant *parameter, GsShell *shell)
 {
 	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
+	const gchar *action_name, *provider_id;
 	GsAuth *auth;
 
-	auth = GS_AUTH (g_object_get_data (G_OBJECT (button), "auth"));
+	action_name = g_action_get_name (G_ACTION (action));
+	g_return_if_fail (g_str_has_prefix (action_name, "signin-"));
+	provider_id = action_name + strlen ("signin-");
+
+	auth = gs_plugin_loader_get_auth_by_id (priv->plugin_loader, provider_id);
+	g_return_if_fail (auth != NULL);
+
 	gs_page_authenticate (priv->page_last, NULL,
 			      gs_auth_get_provider_id (auth),
 			      priv->cancellable,
@@ -688,103 +668,56 @@ gs_shell_logout_cb (GObject *source,
 }
 
 static void
-gs_shell_signout_button_cb (GtkButton *button, GsShell *shell)
+signout_activated_cb (GSimpleAction *action, GVariant *parameter, GsShell *shell)
 {
 	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
+	const gchar *action_name, *provider_id;
 	g_autoptr(GsPluginJob) plugin_job = NULL;
+	GsAuth *auth;
+
+	action_name = g_action_get_name (G_ACTION (action));
+	g_return_if_fail (g_str_has_prefix (action_name, "signout-"));
+	provider_id = action_name + strlen ("signout-");
+
+	auth = gs_plugin_loader_get_auth_by_id (priv->plugin_loader, provider_id);
+	g_return_if_fail (auth != NULL);
 
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_AUTH_LOGOUT,
 					 "interactive", TRUE,
-					 "auth", g_object_get_data (G_OBJECT (button), "auth"),
+					 "auth", auth,
 					 NULL);
 
 	gs_plugin_loader_job_process_async (priv->plugin_loader, plugin_job,
 					    priv->cancellable,
 					    gs_shell_logout_cb,
 					    shell);
-
 }
 
 static void
-add_buttons_for_auth (GsShell *shell, GsAuth *auth)
-{
-	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
-	GtkWidget *account_box;
-	gboolean logged_in;
-	GtkWidget *signin_button;
-	GtkWidget *signout_button;
-	g_autofree gchar *signout_label = NULL;
-	g_autofree gchar *signin_label = NULL;
-
-	account_box = GTK_WIDGET (gtk_builder_get_object (priv->builder, "account_box"));
-	logged_in = gs_auth_has_flag (auth, GS_AUTH_FLAG_VALID);
-	signin_button = gtk_model_button_new ();
-	signout_button = gtk_model_button_new ();
-
-	signout_label = g_strdup_printf (_("Sign out from %s"),
-					 gs_auth_get_provider_name (auth));
-	if (logged_in)
-		signin_label = g_strdup_printf (_("Signed in into %s as %s"),
-						gs_auth_get_provider_name (auth),
-						gs_auth_get_username (auth));
-	else
-		signin_label = g_strdup_printf (_("Sign in to %s…"),
-						gs_auth_get_provider_name (auth));
-
-	g_object_set (signin_button,
-		      "text", signin_label,
-		      "sensitive", !logged_in, NULL);
-	g_object_set_data (G_OBJECT (signin_button), "auth", auth);
-
-	g_object_set (signout_button,
-		      "text", signout_label,
-		      "sensitive", logged_in, NULL);
-	g_object_set_data (G_OBJECT (signout_button), "auth", auth);
-
-	g_signal_connect (signin_button, "clicked",
-			  G_CALLBACK (gs_shell_signin_button_cb), shell);
-	g_signal_connect (signout_button, "clicked",
-			  G_CALLBACK (gs_shell_signout_button_cb), shell);
-
-	gtk_widget_show (signin_button);
-	gtk_widget_show (signout_button);
-	gtk_box_pack_start (GTK_BOX (account_box), signin_button, TRUE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (account_box), signout_button, TRUE, TRUE, 0);
-}
-
-static void
-account_button_clicked_cb (GtkButton *button, GsShell *shell)
+menu_button_clicked_cb (GtkButton *button, GsShell *shell)
 {
 	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
 	GPtrArray *auth_array;
-	GtkWidget *account_popover;
-	GtkWidget *account_box;
-	g_autoptr(GList) children = NULL;
 
 	auth_array = gs_plugin_loader_get_auths (priv->plugin_loader);
-	account_popover = GTK_WIDGET (gtk_builder_get_object (priv->builder, "account_popover"));
-	account_box = GTK_WIDGET (gtk_builder_get_object (priv->builder, "account_box"));
-
-	/* Remove existing buttons... */
-	children = gtk_container_get_children (GTK_CONTAINER (account_box));
-	for (GList *l = children; l != NULL; l = l->next)
-		gtk_container_remove (GTK_CONTAINER (account_box), GTK_WIDGET (l->data));
-
-	/* Add new ones... */
 	for (guint i = 0; i < auth_array->len; i++) {
 		GsAuth *auth = g_ptr_array_index (auth_array, i);
-		add_buttons_for_auth (shell, auth);
+		gboolean logged_in;
+		g_autofree gchar *signin_action_name = NULL;
+		GSimpleAction *signin_action;
+		g_autofree gchar *signout_action_name = NULL;
+		GSimpleAction *signout_action;
 
-		/* Add sepeartor between each block */
-		if (i < auth_array->len - 1) {
-			GtkWidget *seprator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-			gtk_widget_show (seprator);
-			gtk_box_pack_start (GTK_BOX (account_box), seprator, TRUE, TRUE, 0);
-		}
+		logged_in = gs_auth_has_flag (auth, GS_AUTH_FLAG_VALID);
+
+		signin_action_name = g_strdup_printf ("signin-%s", gs_auth_get_provider_id (auth));
+		signin_action = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (priv->auth_actions), signin_action_name));
+		g_simple_action_set_enabled (signin_action, !logged_in);
+
+		signout_action_name = g_strdup_printf ("signout-%s", gs_auth_get_provider_id (auth));
+		signout_action = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (priv->auth_actions), signout_action_name));
+		g_simple_action_set_enabled (signout_action, logged_in);
 	}
-
-	gtk_popover_set_relative_to (GTK_POPOVER (account_popover), GTK_WIDGET (button));
-	gtk_popover_popup (GTK_POPOVER (account_popover));
 }
 
 static gboolean
@@ -838,6 +771,7 @@ static gboolean
 main_window_closed_cb (GtkWidget *dialog, GdkEvent *event, gpointer user_data)
 {
 	GsShell *shell = user_data;
+	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
 
 	/* hide any notifications */
 	g_application_withdraw_notification (g_application_get_default (),
@@ -845,11 +779,25 @@ main_window_closed_cb (GtkWidget *dialog, GdkEvent *event, gpointer user_data)
 	g_application_withdraw_notification (g_application_get_default (),
 					     "install-resources");
 
+	/* When the window is closed, reset the initial mode to overview */
+	priv->mode = GS_SHELL_MODE_OVERVIEW;
+
 	gs_shell_clean_back_entry_stack (shell);
 	gtk_widget_hide (dialog);
 	return TRUE;
 }
 
+static void
+gs_shell_reload_cb (GsPluginLoader *plugin_loader, GsShell *shell)
+{
+	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
+	g_autoptr(GList) keys = g_hash_table_get_keys (priv->pages);
+	for (GList *l = keys; l != NULL; l = l->next) {
+		GsPage *page = GS_PAGE (g_hash_table_lookup (priv->pages, l->data));
+		gs_page_reload (page);
+	}
+}
+
 static void
 gs_shell_main_window_mapped_cb (GtkWidget *widget, GsShell *shell)
 {
@@ -1029,7 +977,6 @@ gs_shell_show_event_refresh (GsShell *shell, GsPluginEvent *event)
 	GsShellEventButtons buttons = GS_SHELL_EVENT_BUTTON_NONE;
 	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
 	const GError *error = gs_plugin_event_get_error (event);
-	GsPluginAction action = gs_plugin_event_get_action (event);
 	g_autofree gchar *str_origin = NULL;
 	g_autoptr(GString) str = g_string_new (NULL);
 
@@ -1099,13 +1046,8 @@ gs_shell_show_event_refresh (GsShell *shell, GsPluginEvent *event)
 		/* non-interactive generic */
 		if (!gs_plugin_event_has_flag (event, GS_PLUGIN_EVENT_FLAG_INTERACTIVE))
 			return FALSE;
-		if (action == GS_PLUGIN_ACTION_DOWNLOAD) {
-			/* TRANSLATORS: failure text for the in-app notification */
-			g_string_append (str, _("Unable to download updates"));
-		} else {
-			/* TRANSLATORS: failure text for the in-app notification */
-			g_string_append (str, _("Unable to get list of updates"));
-		}
+		/* TRANSLATORS: failure text for the in-app notification */
+		g_string_append (str, _("Unable to get list of updates"));
 		break;
 	}
 	if (str->len == 0)
@@ -1880,11 +1822,11 @@ gs_shell_show_event (GsShell *shell, GsPluginEvent *event)
 	action = gs_plugin_event_get_action (event);
 	switch (action) {
 	case GS_PLUGIN_ACTION_REFRESH:
-	case GS_PLUGIN_ACTION_DOWNLOAD:
 		return gs_shell_show_event_refresh (shell, event);
 	case GS_PLUGIN_ACTION_PURCHASE:
 		return gs_shell_show_event_purchase (shell, event);
 	case GS_PLUGIN_ACTION_INSTALL:
+	case GS_PLUGIN_ACTION_DOWNLOAD:
 		return gs_shell_show_event_install (shell, event);
 	case GS_PLUGIN_ACTION_UPDATE:
 		return gs_shell_show_event_update (shell, event);
@@ -1909,9 +1851,9 @@ gs_shell_show_event (GsShell *shell, GsPluginEvent *event)
 static void
 gs_shell_rescan_events (GsShell *shell)
 {
+	GsPluginEvent *event;
 	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
 	GtkWidget *widget;
-	g_autoptr(GsPluginEvent) event = NULL;
 
 	/* find the first active event and show it */
 	event = gs_plugin_loader_get_event_default (priv->plugin_loader);
@@ -1988,6 +1930,24 @@ gs_shell_setup_pages (GsShell *shell)
 	}
 }
 
+static void
+gs_shell_add_about_menu_item (GsShell *shell)
+{
+	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
+	GMenu *primary_menu;
+	g_autoptr(GMenuItem) menu_item = NULL;
+	g_autofree gchar *label = NULL;
+
+	primary_menu = G_MENU (gtk_builder_get_object (priv->builder, "primary_menu"));
+
+	/* TRANSLATORS: this is the menu item that opens the about window, e.g.
+	 * 'About Software' or 'About Application Installer' where the %s is
+	 * the application name chosen by the distro */
+	label = g_strdup_printf (_("About %s"), g_get_application_name ());
+	menu_item = g_menu_item_new (label, "app.about");
+	g_menu_append_item (primary_menu, menu_item);
+}
+
 void
 gs_shell_setup (GsShell *shell, GsPluginLoader *plugin_loader, GCancellable *cancellable)
 {
@@ -1995,10 +1955,13 @@ gs_shell_setup (GsShell *shell, GsPluginLoader *plugin_loader, GCancellable *can
 	GtkWidget *widget;
 	GtkStyleContext *style_context;
 	GsPage *page;
+	GPtrArray *auth_array;
 
 	g_return_if_fail (GS_IS_SHELL (shell));
 
 	priv->plugin_loader = g_object_ref (plugin_loader);
+	g_signal_connect (priv->plugin_loader, "reload",
+			  G_CALLBACK (gs_shell_reload_cb), shell);
 	g_signal_connect_object (priv->plugin_loader, "notify::events",
 				 G_CALLBACK (gs_shell_events_notify_cb),
 				 shell, 0);
@@ -2051,9 +2014,9 @@ gs_shell_setup (GsShell *shell, GsPluginLoader *plugin_loader, GCancellable *can
 	                  shell);
 
 	/* show the account popover when clicking on the account button */
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "account_button"));
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "menu_button"));
 	g_signal_connect (widget, "clicked",
-	                  G_CALLBACK (account_button_clicked_cb),
+	                  G_CALLBACK (menu_button_clicked_cb),
 	                  shell);
 
 	/* setup buttons */
@@ -2131,18 +2094,91 @@ gs_shell_setup (GsShell *shell, GsPluginLoader *plugin_loader, GCancellable *can
 	/* load content */
 	page = GS_PAGE (gtk_builder_get_object (priv->builder, "loading_page"));
 	g_signal_connect (page, "refreshed",
-			  G_CALLBACK (initial_refresh_done), shell);
+			  G_CALLBACK (initial_overview_load_done), shell);
 
 	/* coldplug */
 	gs_shell_rescan_events (shell);
 
-	/* show loading page, which triggers the initial refresh */
-	gs_shell_change_mode (shell, GS_SHELL_MODE_LOADING, NULL, TRUE);
+	/* primary menu */
+	gs_shell_add_about_menu_item (shell);
+
+	/* auth menu */
+	priv->auth_actions = g_simple_action_group_new ();
+	gtk_widget_insert_action_group (GTK_WIDGET (priv->main_window), "auth", G_ACTION_GROUP (priv->auth_actions));
+	auth_array = gs_plugin_loader_get_auths (priv->plugin_loader);
+	for (guint i = 0; i < auth_array->len; i++) {
+		GsAuth *auth = g_ptr_array_index (auth_array, i);
+		GMenu *accounts_menu;
+		gboolean logged_in;
+		g_autoptr(GMenu) auth_menu = NULL;
+		g_autoptr(GSimpleAction) signin_action = NULL;
+		g_autofree gchar *signin_action_name = NULL;
+		g_autoptr(GMenuItem) signin_item = NULL;
+		g_autofree gchar *signin_label = NULL;
+		g_autofree gchar *signin_target = NULL;
+		g_autoptr(GSimpleAction) signout_action = NULL;
+		g_autofree gchar *signout_action_name = NULL;
+		g_autoptr(GMenuItem) signout_item = NULL;
+		g_autofree gchar *signout_label = NULL;
+		g_autofree gchar *signout_target = NULL;
+
+		logged_in = gs_auth_has_flag (auth, GS_AUTH_FLAG_VALID);
+
+		signin_action_name = g_strdup_printf ("signin-%s", gs_auth_get_provider_id (auth));
+		signin_action = g_simple_action_new (signin_action_name, NULL);
+		g_simple_action_set_enabled (signin_action, !logged_in);
+		g_signal_connect (signin_action, "activate", G_CALLBACK (signin_activated_cb), shell);
+		g_action_map_add_action (G_ACTION_MAP (priv->auth_actions), G_ACTION (signin_action));
+
+		signout_action_name = g_strdup_printf ("signout-%s", gs_auth_get_provider_id (auth));
+		signout_action = g_simple_action_new (signout_action_name, NULL);
+		g_simple_action_set_enabled (signout_action, logged_in);
+		g_signal_connect (signout_action, "activate", G_CALLBACK (signout_activated_cb), shell);
+		g_action_map_add_action (G_ACTION_MAP (priv->auth_actions), G_ACTION (signout_action));
+
+		auth_menu = g_menu_new ();
+		accounts_menu = G_MENU (gtk_builder_get_object (priv->builder, "accounts_menu"));
+		g_menu_append_section (accounts_menu, NULL, G_MENU_MODEL (auth_menu));
+
+		if (logged_in) {
+			/* TRANSLATORS: menu item that signs into the named account with a particular username */
+			signin_label = g_strdup_printf (_("Signed in into %s as %s"),
+							gs_auth_get_provider_name (auth),
+							gs_auth_get_username (auth));
+		} else {
+			/* TRANSLATORS: menu item that signs into the named account */
+			signin_label = g_strdup_printf (_("Sign in to %s…"),
+							gs_auth_get_provider_name (auth));
+		}
+		signin_target = g_strdup_printf ("auth.%s", signin_action_name);
+		signin_item = g_menu_item_new (signin_label, signin_target);
+		g_menu_append_item (auth_menu, signin_item);
+
+		/* TRANSLATORS: menu item for signing out from the named account */
+		signout_label = g_strdup_printf (_("Sign out from %s"),
+						 gs_auth_get_provider_name (auth));
+		signout_target = g_strdup_printf ("auth.%s", signout_action_name);
+		signout_item = g_menu_item_new (signout_label, signout_target);
+		g_menu_append_item (auth_menu, signout_item);
+	}
 }
 
 void
 gs_shell_set_mode (GsShell *shell, GsShellMode mode)
 {
+	GsShellPrivate *priv = gs_shell_get_instance_private (shell);
+	guint matched;
+
+	/* if we're loading a different mode at startup then don't wait for
+	 * the overview page to load before showing content */
+	if (mode != GS_SHELL_MODE_OVERVIEW) {
+		GsPage *page = g_hash_table_lookup (priv->pages, "overview");
+		matched = g_signal_handlers_disconnect_by_func (page,
+								initial_overview_load_done,
+								shell);
+		if (matched > 0)
+			g_signal_emit (shell, signals[SIGNAL_LOADED], 0);
+	}
 	gs_shell_change_mode (shell, mode, NULL, TRUE);
 }
 
@@ -2312,6 +2348,7 @@ gs_shell_dispose (GObject *object)
 	g_clear_pointer (&priv->pages, g_hash_table_unref);
 	g_clear_pointer (&priv->events_info_uri, g_free);
 	g_clear_pointer (&priv->modal_dialogs, g_ptr_array_unref);
+	g_clear_object (&priv->auth_actions);
 
 	G_OBJECT_CLASS (gs_shell_parent_class)->dispose (object);
 }
