@@ -53,8 +53,15 @@ struct _GsUpdateMonitor {
 
 G_DEFINE_TYPE (GsUpdateMonitor, gs_update_monitor, G_TYPE_OBJECT)
 
+typedef enum {
+	GS_UPDATE_MONITOR_MODE_NONE		= 0,
+	GS_UPDATE_MONITOR_MODE_DO_AUTOUPDATES	= 1 << 0,
+	GS_UPDATE_MONITOR_MODE_LAST
+} GsUpdateMonitorMode;
+
 typedef struct {
 	GsUpdateMonitor		*monitor;
+	GsUpdateMonitorMode	 mode;
 } DownloadUpdatesData;
 
 static void
@@ -216,14 +223,14 @@ _build_autoupdated_notification (GsUpdateMonitor *monitor, GsAppList *list)
 			/* TRANSLATORS: apps were auto-updated and restart is required */
 			title = g_strdup_printf (ngettext ("%u Application Updated — Restart Required",
 			                                   "%u Applications Updated — Restart Required",
-			                                   gs_app_list_length (list_apps)),
-			                         gs_app_list_length (list_apps));
+			                                   gs_app_list_length (list)),
+			                         gs_app_list_length (list));
 		} else {
 			/* TRANSLATORS: apps were auto-updated */
 			title = g_strdup_printf (ngettext ("%u Application Updated",
 			                                   "%u Applications Updated",
-			                                   gs_app_list_length (list_apps)),
-			                         gs_app_list_length (list_apps));
+			                                   gs_app_list_length (list)),
+			                         gs_app_list_length (list));
 		}
 	}
 
@@ -325,8 +332,7 @@ download_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 	GsUpdateMonitor *monitor = GS_UPDATE_MONITOR (data);
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GsAppList) list = NULL;
-	g_autoptr(GsAppList) update_online = NULL;
-	g_autoptr(GsAppList) update_offline = NULL;
+	g_autoptr(GsAppList) list2 = NULL;
 
 	/* get result */
 	list = gs_plugin_loader_job_process_finish (GS_PLUGIN_LOADER (object), res, &error);
@@ -336,23 +342,19 @@ download_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 		return;
 	}
 
-	update_online = gs_app_list_new ();
-	update_offline = gs_app_list_new ();
+	/* install any apps that can be installed LIVE */
+	list2 = gs_app_list_new ();
 	for (guint i = 0; i < gs_app_list_length (list); i++) {
 		GsApp *app = gs_app_list_index (list, i);
 		if (_should_auto_update (app)) {
 			g_debug ("auto-updating %s", gs_app_get_unique_id (app));
-			gs_app_list_add (update_online, app);
-		} else {
-			gs_app_list_add (update_offline, app);
+			gs_app_list_add (list2, app);
 		}
 	}
-
-	/* install any apps that can be installed LIVE */
-	if (gs_app_list_length (update_online) > 0) {
+	if (gs_app_list_length (list2) > 0) {
 		g_autoptr(GsPluginJob) plugin_job = NULL;
 		plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_UPDATE,
-						 "list", update_online,
+						 "list", list2,
 						 NULL);
 		gs_plugin_loader_job_process_async (monitor->plugin_loader,
 						    plugin_job,
@@ -360,14 +362,6 @@ download_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 						    update_finished_cb,
 						    monitor);
 	}
-
-	/* show a notification for offline updates */
-	if (gs_app_list_length (update_offline) > 0) {
-		if (has_important_updates (update_offline) ||
-		    no_updates_for_a_week (monitor)) {
-			notify_offline_update_available (monitor);
-		}
-	}
 }
 
 static void
@@ -414,7 +408,8 @@ get_updates_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 	g_debug ("got %u updates", gs_app_list_length (apps));
 
 	/* download any updates if auto-updates are turned on */
-	if (g_settings_get_boolean (monitor->settings, "download-updates")) {
+	if (download_updates_data->mode == GS_UPDATE_MONITOR_MODE_DO_AUTOUPDATES &&
+	    g_settings_get_boolean (monitor->settings, "download-updates")) {
 		g_autoptr(GsPluginJob) plugin_job = NULL;
 		plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_DOWNLOAD,
 						 "list", apps,
@@ -425,12 +420,12 @@ get_updates_finished_cb (GObject *object, GAsyncResult *res, gpointer data)
 						    monitor->cancellable,
 						    download_finished_cb,
 						    monitor);
-	} else {
-		/* notify immediately if auto-updates are turned off */
-		if (has_important_updates (apps) ||
-		    no_updates_for_a_week (monitor)) {
-			notify_offline_update_available (monitor);
-		}
+		return;
+	}
+
+	if (has_important_updates (apps) ||
+	    no_updates_for_a_week (monitor)) {
+		notify_offline_update_available (monitor);
 	}
 }
 
@@ -556,7 +551,7 @@ get_upgrades_finished_cb (GObject *object,
 }
 
 static void
-get_updates (GsUpdateMonitor *monitor)
+get_updates (GsUpdateMonitor *monitor, GsUpdateMonitorMode mode)
 {
 	g_autoptr(GsPluginJob) plugin_job = NULL;
 	g_autoptr(DownloadUpdatesData) download_updates_data = NULL;
@@ -569,6 +564,7 @@ get_updates (GsUpdateMonitor *monitor)
 
 	download_updates_data = g_slice_new0 (DownloadUpdatesData);
 	download_updates_data->monitor = g_object_ref (monitor);
+	download_updates_data->mode = mode;
 
 	/* NOTE: this doesn't actually do any network access */
 	g_debug ("Getting updates");
@@ -586,7 +582,7 @@ get_updates (GsUpdateMonitor *monitor)
 void
 gs_update_monitor_autoupdate (GsUpdateMonitor *monitor)
 {
-	get_updates (monitor);
+	get_updates (monitor, GS_UPDATE_MONITOR_MODE_DO_AUTOUPDATES);
 }
 
 static void
@@ -650,7 +646,7 @@ refresh_cache_finished_cb (GObject *object,
 	g_settings_set (monitor->settings, "check-timestamp", "x",
 	                g_date_time_to_unix (now));
 
-	get_updates (monitor);
+	get_updates (monitor, GS_UPDATE_MONITOR_MODE_DO_AUTOUPDATES);
 }
 
 typedef enum {
@@ -829,6 +825,14 @@ network_available_notify_cb (GsPluginLoader *plugin_loader,
 	check_updates (monitor);
 }
 
+static void
+updates_changed_cb (GsPluginLoader *plugin_loader, GsUpdateMonitor *monitor)
+{
+	/* when the list of downloaded-and-ready-to-go updates changes get the
+	 * new list and perhaps show/hide the notification */
+	get_updates (monitor, GS_UPDATE_MONITOR_MODE_NONE);
+}
+
 static void
 get_updates_historical_cb (GObject *object, GAsyncResult *res, gpointer data)
 {
@@ -1086,14 +1090,10 @@ gs_update_monitor_dispose (GObject *object)
 		monitor->network_changed_handler = 0;
 	}
 
-	if (monitor->cancellable != NULL) {
-		g_cancellable_cancel (monitor->cancellable);
-		g_clear_object (&monitor->cancellable);
-	}
-	if (monitor->network_cancellable != NULL) {
-		g_cancellable_cancel (monitor->network_cancellable);
-		g_clear_object (&monitor->network_cancellable);
-	}
+	g_cancellable_cancel (monitor->cancellable);
+	g_clear_object (&monitor->cancellable);
+	g_cancellable_cancel (monitor->network_cancellable);
+	g_clear_object (&monitor->network_cancellable);
 
 	stop_updates_check (monitor);
 	stop_upgrades_check (monitor);
@@ -1111,6 +1111,9 @@ gs_update_monitor_dispose (GObject *object)
 		monitor->cleanup_notifications_id = 0;
 	}
 	if (monitor->plugin_loader != NULL) {
+		g_signal_handlers_disconnect_by_func (monitor->plugin_loader,
+		                                      updates_changed_cb,
+		                                      monitor);
 		g_signal_handlers_disconnect_by_func (monitor->plugin_loader,
 						      network_available_notify_cb,
 						      monitor);
@@ -1151,6 +1154,8 @@ gs_update_monitor_new (GsApplication *application)
 	g_application_hold (monitor->application);
 
 	monitor->plugin_loader = gs_application_get_plugin_loader (application);
+	g_signal_connect (monitor->plugin_loader, "updates-changed",
+			  G_CALLBACK (updates_changed_cb), monitor);
 	g_signal_connect (monitor->plugin_loader, "notify::allow-updates",
 			  G_CALLBACK (allow_updates_notify_cb), monitor);
 	g_signal_connect (monitor->plugin_loader, "notify::network-available",
