@@ -42,6 +42,8 @@
 
 #include <langinfo.h>
 
+#define GS_REFRESH_MIN_AGE 10 /* sec */
+
 typedef enum {
 	GS_UPDATES_PAGE_FLAG_NONE		= 0,
 	GS_UPDATES_PAGE_FLAG_HAS_UPDATES	= 1 << 0,
@@ -279,6 +281,11 @@ gs_updates_page_last_checked_time_string (GsUpdatesPage *self)
 static const gchar *
 gs_updates_page_get_state_string (GsPluginStatus status)
 {
+	if (status == GS_PLUGIN_STATUS_DOWNLOADING) {
+		/* TRANSLATORS: the updates are being downloaded */
+		return _("Downloading new updates…");
+	}
+
 	/* TRANSLATORS: the update panel is doing *something* vague */
 	return _("Looking for new updates…");
 }
@@ -376,7 +383,7 @@ gs_updates_page_update_ui_state (GsUpdatesPage *self)
 	case GS_UPDATES_PAGE_STATE_MANAGED:
 		gtk_widget_hide (self->button_refresh);
 		break;
-	case GS_UPDATES_PAGE_STATE_IDLE:
+	default:
 		gtk_image_set_from_icon_name (GTK_IMAGE (gtk_button_get_image (GTK_BUTTON (self->button_refresh))),
 					      "view-refresh-symbolic", GTK_ICON_SIZE_MENU);
 		if (self->result_flags != GS_UPDATES_PAGE_FLAG_NONE) {
@@ -388,14 +395,6 @@ gs_updates_page_update_ui_state (GsUpdatesPage *self)
 			gtk_widget_set_visible (self->button_refresh, allow_mobile_refresh);
 		}
 		break;
-	case GS_UPDATES_PAGE_STATE_FAILED:
-		gtk_image_set_from_icon_name (GTK_IMAGE (gtk_button_get_image (GTK_BUTTON (self->button_refresh))),
-					      "view-refresh-symbolic", GTK_ICON_SIZE_MENU);
-		gtk_widget_show (self->button_refresh);
-		break;
-	default:
-		g_assert_not_reached ();
-		break;
 	}
 	gtk_widget_set_sensitive (self->button_refresh,
 				  gs_plugin_loader_get_network_available (self->plugin_loader));
@@ -527,6 +526,7 @@ gs_updates_page_get_updates_cb (GsPluginLoader *plugin_loader,
 		gs_updates_page_clear_flag (self, GS_UPDATES_PAGE_FLAG_HAS_UPDATES);
 		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED))
 			g_warning ("updates-shell: failed to get updates: %s", error->message);
+		gs_utils_error_strip_unique_id (error);
 		gtk_label_set_label (GTK_LABEL (self->label_updates_failed),
 				     error->message);
 		gs_updates_page_set_state (self, GS_UPDATES_PAGE_STATE_FAILED);
@@ -543,6 +543,12 @@ gs_updates_page_get_updates_cb (GsPluginLoader *plugin_loader,
 		gs_updates_section_add_app (GS_UPDATES_SECTION (self->sections[section]), app);
 	}
 
+	/* invalidate the headers */
+	for (guint i = 0; i < GS_UPDATES_SECTION_KIND_LAST; i++) {
+		if (self->sections[i] != NULL)
+			gtk_list_box_invalidate_headers (self->sections[i]);
+	}
+
 	/* update the counter in headerbar */
 	refresh_headerbar_updates_counter (self);
 
@@ -666,13 +672,12 @@ gs_updates_page_load (GsUpdatesPage *self)
 	refine_flags = GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_SIZE |
 		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_DETAILS |
-		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_PROVENANCE |
 		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION;
 	gs_updates_page_set_state (self, GS_UPDATES_PAGE_STATE_ACTION_GET_UPDATES);
 	self->action_cnt++;
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_GET_UPDATES,
-					 "interactive", TRUE,
 					 "refine-flags", refine_flags,
+					 "dedupe-flags", GS_APP_LIST_FILTER_FLAG_NONE,
 					 NULL);
 	gs_plugin_loader_job_process_async (self->plugin_loader, plugin_job,
 					    self->cancellable,
@@ -683,7 +688,6 @@ gs_updates_page_load (GsUpdatesPage *self)
 	g_object_unref (plugin_job);
 	app = gs_plugin_loader_get_system_app (self->plugin_loader);
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_REFINE,
-					 "interactive", TRUE,
 					 "app", app,
 					 "refine-flags", refine_flags,
 					 NULL);
@@ -697,7 +701,6 @@ gs_updates_page_load (GsUpdatesPage *self)
 		refine_flags |= GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPGRADE_REMOVED;
 		g_object_unref (plugin_job);
 		plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_GET_DISTRO_UPDATES,
-						 "interactive", TRUE,
 						 "refine-flags", refine_flags,
 						 NULL);
 		gs_plugin_loader_job_process_async (self->plugin_loader,
@@ -713,12 +716,6 @@ static void
 gs_updates_page_reload (GsPage *page)
 {
 	GsUpdatesPage *self = GS_UPDATES_PAGE (page);
-
-	if (self->state == GS_UPDATES_PAGE_STATE_ACTION_REFRESH) {
-		g_debug ("ignoring reload as refresh is already in progress");
-		return;
-	}
-
 	gs_updates_page_invalidate (self);
 	gs_updates_page_load (self);
 }
@@ -738,6 +735,8 @@ gs_updates_page_switch_to (GsPage *page,
 
 	widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "buttonbox_main"));
 	gtk_widget_show (widget);
+	widget = GTK_WIDGET (gtk_builder_get_object (self->builder, "menu_button"));
+	gtk_widget_show (widget);
 
 	gtk_widget_set_visible (self->button_refresh, TRUE);
 
@@ -780,6 +779,7 @@ gs_updates_page_refresh_cb (GsPluginLoader *plugin_loader,
 			return;
 		}
 		g_warning ("failed to refresh: %s", error->message);
+		gs_utils_error_strip_unique_id (error);
 		gtk_label_set_label (GTK_LABEL (self->label_updates_failed),
 				     error->message);
 		gs_updates_page_set_state (self, GS_UPDATES_PAGE_STATE_FAILED);
@@ -804,15 +804,13 @@ gs_updates_page_get_new_updates (GsUpdatesPage *self)
 	/* force a check for updates and download */
 	gs_updates_page_set_state (self, GS_UPDATES_PAGE_STATE_ACTION_REFRESH);
 
-	if (self->cancellable_refresh != NULL) {
-		g_cancellable_cancel (self->cancellable_refresh);
-		g_object_unref (self->cancellable_refresh);
-	}
+	g_cancellable_cancel (self->cancellable_refresh);
+	g_clear_object (&self->cancellable_refresh);
 	self->cancellable_refresh = g_cancellable_new ();
 
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_REFRESH,
 					 "interactive", TRUE,
-					 "age", (guint64) 1,
+					 "age", (guint64) GS_REFRESH_MIN_AGE,
 					 NULL);
 	gs_plugin_loader_job_process_async (self->plugin_loader, plugin_job,
 					    self->cancellable_refresh,
@@ -887,8 +885,7 @@ gs_updates_page_button_refresh_cb (GtkWidget *widget,
 		gs_updates_page_get_new_updates (self);
 
 	/* expensive network connection */
-	} else if (gs_plugin_loader_get_network_available (self->plugin_loader) &&
-	           gs_plugin_loader_get_network_metered (self->plugin_loader)) {
+	} else if (gs_plugin_loader_get_network_metered (self->plugin_loader)) {
 		if (self->has_agreed_to_mobile_data) {
 			gs_updates_page_get_new_updates (self);
 			return;
@@ -1252,9 +1249,8 @@ gs_updates_page_status_changed_cb (GsPluginLoader *plugin_loader,
 	switch (status) {
 	case GS_PLUGIN_STATUS_INSTALLING:
 	case GS_PLUGIN_STATUS_REMOVING:
-		if (app == NULL ||
-		    (gs_app_get_kind (app) != AS_APP_KIND_OS_UPGRADE &&
-		     gs_app_get_id (app) != NULL)) {
+		if (gs_app_get_kind (app) != AS_APP_KIND_OS_UPGRADE &&
+		    gs_app_get_id (app) != NULL) {
 			/* if we do a install or remove then make sure all new
 			 * packages are downloaded */
 			gs_updates_page_invalidate_downloaded_upgrade (self);
@@ -1308,9 +1304,8 @@ gs_updates_page_setup (GsPage *page,
 						    self->sizegroup_desc,
 						    self->sizegroup_button,
 						    self->sizegroup_header);
-		gtk_box_pack_start (GTK_BOX (self->updates_box),
-				    GTK_WIDGET (self->sections[i]),
-				    TRUE, TRUE, 0);
+		gtk_widget_set_vexpand (GTK_WIDGET (self->sections[i]), FALSE);
+		gtk_container_add (GTK_CONTAINER (self->updates_box), GTK_WIDGET (self->sections[i]));
 	}
 
 	self->shell = shell;
@@ -1354,16 +1349,20 @@ gs_updates_page_setup (GsPage *page,
 
 	/* This label indicates that the update check is in progress */
 	self->header_checking_label = gtk_label_new (_("Checking…"));
-	gtk_box_pack_end (GTK_BOX (self->header_start_box), self->header_checking_label, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (self->header_start_box), self->header_checking_label);
+	gtk_container_child_set(GTK_CONTAINER (self->header_start_box), self->header_checking_label,
+				"pack-type", GTK_PACK_END, NULL);
 	self->header_spinner_start = gtk_spinner_new ();
-	gtk_box_pack_end (GTK_BOX (self->header_start_box), self->header_spinner_start, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (self->header_start_box), self->header_spinner_start);
+	gtk_container_child_set (GTK_CONTAINER (self->header_start_box), self->header_spinner_start,
+				 "pack-type", GTK_PACK_END, NULL);
 
 	/* setup update details window */
 	self->button_refresh = gtk_button_new_from_icon_name ("view-refresh-symbolic", GTK_ICON_SIZE_MENU);
 	accessible = gtk_widget_get_accessible (self->button_refresh);
 	if (accessible != NULL)
 		atk_object_set_name (accessible, _("Check for updates"));
-	gtk_box_pack_start (GTK_BOX (self->header_start_box), self->button_refresh, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (self->header_start_box), self->button_refresh);
 	g_signal_connect (self->button_refresh, "clicked",
 			  G_CALLBACK (gs_updates_page_button_refresh_cb),
 			  self);
@@ -1393,14 +1392,10 @@ gs_updates_page_dispose (GObject *object)
 {
 	GsUpdatesPage *self = GS_UPDATES_PAGE (object);
 
-	if (self->cancellable_refresh != NULL) {
-		g_cancellable_cancel (self->cancellable_refresh);
-		g_clear_object (&self->cancellable_refresh);
-	}
-	if (self->cancellable_upgrade_download != NULL) {
-		g_cancellable_cancel (self->cancellable_upgrade_download);
-		g_clear_object (&self->cancellable_upgrade_download);
-	}
+	g_cancellable_cancel (self->cancellable_refresh);
+	g_clear_object (&self->cancellable_refresh);
+	g_cancellable_cancel (self->cancellable_upgrade_download);
+	g_clear_object (&self->cancellable_upgrade_download);
 
 	for (guint i = 0; i < GS_UPDATES_SECTION_KIND_LAST; i++) {
 		if (self->sections[i] != NULL) {
