@@ -45,11 +45,9 @@ struct _GsUpdatesSection
 	GtkSizeGroup		*sizegroup_desc;
 	GtkSizeGroup		*sizegroup_button;
 	GtkSizeGroup		*sizegroup_header;
-	GtkWidget		*button_download;
-	GtkWidget		*button_update;
+	GtkWidget		*button;
 	GtkWidget		*button_cancel;
 	GtkStack		*button_stack;
-	GtkWidget		*section_header;
 };
 
 G_DEFINE_TYPE (GsUpdatesSection, gs_updates_section, GTK_TYPE_LIST_BOX)
@@ -69,31 +67,12 @@ _app_row_button_clicked_cb (GsAppRow *app_row, GsUpdatesSection *self)
 	gs_page_update_app (GS_PAGE (self->page), app, gs_app_get_cancellable (app));
 }
 
-static void
-_row_unrevealed_cb (GObject *row, GParamSpec *pspec, gpointer data)
-{
-	GtkWidget *list;
-
-	list = gtk_widget_get_parent (GTK_WIDGET (row));
-	if (list == NULL)
-		return;
-	gtk_container_remove (GTK_CONTAINER (list), GTK_WIDGET (row));
-}
-
-static void
-_unreveal_row (GsAppRow *app_row)
-{
-	gs_app_row_unreveal (app_row);
-	g_signal_connect (app_row, "unrevealed",
-	                  G_CALLBACK (_row_unrevealed_cb), NULL);
-}
-
 static void
 _app_state_notify_cb (GsApp *app, GParamSpec *pspec, gpointer user_data)
 {
 	if (gs_app_get_state (app) == AS_APP_STATE_INSTALLED) {
 		GsAppRow *app_row = GS_APP_ROW (user_data);
-		_unreveal_row (app_row);
+		gs_app_row_unreveal (app_row);
 	}
 }
 
@@ -130,6 +109,14 @@ gs_updates_section_remove_all (GsUpdatesSection *self)
 		GtkWidget *w = GTK_WIDGET (l->data);
 		gtk_container_remove (GTK_CONTAINER (self), w);
 	}
+
+	/* the following are set in _build_section_header(); clear these so
+	 * that they don't become dangling pointers once all items are removed
+	 * from self->list */
+	self->button = NULL;
+	self->button_cancel = NULL;
+	self->button_stack = NULL;
+
 	gs_app_list_remove_all (self->list);
 	gtk_widget_hide (GTK_WIDGET (self));
 }
@@ -249,20 +236,6 @@ _reboot_failed_cb (GObject *source, GAsyncResult *res, gpointer user_data)
 					    self);
 }
 
-static gboolean
-_all_offline_updates_downloaded (GsUpdatesSection *self)
-{
-	/* use the download size to figure out what is downloaded and what not */
-	for (guint i = 0; i < gs_app_list_length (self->list); i++) {
-		GsApp *app = gs_app_list_index (self->list, i);
-		guint64 size = gs_app_get_size_download (app);
-		if (size != 0)
-			return FALSE;
-	}
-
-	return TRUE;
-}
-
 static void
 _update_buttons (GsUpdatesSection *self)
 {
@@ -277,21 +250,17 @@ _update_buttons (GsUpdatesSection *self)
 
 	if (self->kind == GS_UPDATES_SECTION_KIND_OFFLINE_FIRMWARE ||
 	    self->kind == GS_UPDATES_SECTION_KIND_OFFLINE) {
-		if (_all_offline_updates_downloaded (self))
-			gtk_stack_set_visible_child_name (self->button_stack, "update");
-		else
-			gtk_stack_set_visible_child_name (self->button_stack, "download");
-
+		gtk_stack_set_visible_child_name (self->button_stack, "update");
 		gtk_widget_show (GTK_WIDGET (self->button_stack));
-		/* TRANSLATORS: This is the button for installing all
+		/* TRANSLATORS: This is the button for upgrading all
 		 * offline updates */
-		gtk_button_set_label (GTK_BUTTON (self->button_update), _("Restart & Update"));
+		gtk_button_set_label (GTK_BUTTON (self->button), _("Restart & Update"));
 	} else if (self->kind == GS_UPDATES_SECTION_KIND_ONLINE) {
 		gtk_stack_set_visible_child_name (self->button_stack, "update");
 		gtk_widget_show (GTK_WIDGET (self->button_stack));
 		/* TRANSLATORS: This is the button for upgrading all
 		 * online-updatable applications */
-		gtk_button_set_label (GTK_BUTTON (self->button_update), _("Update All"));
+		gtk_button_set_label (GTK_BUTTON (self->button), _("Update All"));
 	} else {
 		gtk_widget_hide (GTK_WIDGET (self->button_stack));
 	}
@@ -355,42 +324,19 @@ _button_cancel_clicked_cb (GtkButton *button, GsUpdatesSection *self)
 	_update_buttons (self);
 }
 
-static void
-_download_finished_cb (GObject *object, GAsyncResult *res, gpointer user_data)
+static GsAppList *
+_get_listbox_apps (GsUpdatesSection *self)
 {
-	g_autoptr(GsUpdatesSection) self = (GsUpdatesSection *) user_data;
-	g_autoptr(GError) error = NULL;
-	g_autoptr(GsAppList) list = NULL;
+	g_autoptr(GsAppList) apps = gs_app_list_new ();
+	g_autoptr(GList) listbox_children = NULL;
 
-	/* get result */
-	list = gs_plugin_loader_job_process_finish (GS_PLUGIN_LOADER (object), res, &error);
-	if (list == NULL) {
-		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED))
-			g_warning ("failed to download updates: %s", error->message);
+	listbox_children = gtk_container_get_children (GTK_CONTAINER (self));
+	for (GList *l = listbox_children; l; l = l->next) {
+		GsAppRow *app_row = GS_APP_ROW (l->data);
+		gs_app_list_add (apps, gs_app_row_get_app (app_row));
 	}
 
-	g_clear_object (&self->cancellable);
-	_update_buttons (self);
-}
-
-static void
-_button_download_clicked_cb (GtkButton *button, GsUpdatesSection *self)
-{
-	g_autoptr(GError) error = NULL;
-	g_autoptr(GCancellable) cancellable = g_cancellable_new ();
-	g_autoptr(GsPluginJob) plugin_job = NULL;
-
-	g_set_object (&self->cancellable, cancellable);
-	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_DOWNLOAD,
-					 "list", self->list,
-					 "refine-flags", GS_PLUGIN_REFINE_FLAGS_REQUIRE_SIZE,
-					 "interactive", TRUE,
-					 NULL);
-	gs_plugin_loader_job_process_async (self->plugin_loader, plugin_job,
-					    self->cancellable,
-					    (GAsyncReadyCallback) _download_finished_cb,
-					    g_object_ref (self));
-	_update_buttons (self);
+	return g_steal_pointer (&apps);
 }
 
 static void
@@ -400,12 +346,17 @@ _button_update_all_clicked_cb (GtkButton *button, GsUpdatesSection *self)
 	g_autoptr(GCancellable) cancellable = g_cancellable_new ();
 	g_autoptr(GsPluginJob) plugin_job = NULL;
 	GsUpdatesSectionUpdateHelper *helper = g_new0 (GsUpdatesSectionUpdateHelper, 1);
+	g_autoptr(GsAppList) sorted_list = NULL;
+
+	/* instead of using self->list, get the already-sorted apps back from
+	 * the listbox to be able to update them in display order */
+	sorted_list = _get_listbox_apps (self);
 
 	helper->self = g_object_ref (self);
 
 	/* look at each app in turn */
-	for (guint i = 0; i < gs_app_list_length (self->list); i++) {
-		GsApp *app = gs_app_list_index (self->list, i);
+	for (guint i = 0; i < gs_app_list_length (sorted_list); i++) {
+		GsApp *app = gs_app_list_index (sorted_list, i);
 		if (gs_app_get_state (app) == AS_APP_STATE_UPDATABLE)
 			helper->do_reboot = TRUE;
 		if (gs_app_has_quirk (app, AS_APP_QUIRK_NEEDS_REBOOT))
@@ -414,7 +365,7 @@ _button_update_all_clicked_cb (GtkButton *button, GsUpdatesSection *self)
 
 	g_set_object (&self->cancellable, cancellable);
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_UPDATE,
-					 "list", self->list,
+					 "list", sorted_list,
 					 "interactive", TRUE,
 					 NULL);
 	gs_plugin_loader_job_process_async (self->plugin_loader, plugin_job,
@@ -454,11 +405,13 @@ _build_section_header (GsUpdatesSection *self)
 
 	/* create header */
 	header = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 3);
+	gtk_size_group_add_widget (self->sizegroup_header, header);
 	context = gtk_widget_get_style_context (header);
 	gtk_style_context_add_class (context, "app-listbox-header");
 
 	/* put label into the header */
-	gtk_box_pack_start (GTK_BOX (header), label, TRUE, TRUE, 0);
+	gtk_widget_set_hexpand (label, TRUE);
+	gtk_container_add (GTK_CONTAINER (header), label);
 	gtk_widget_set_visible (label, TRUE);
 	gtk_widget_set_margin_start (label, 6);
 	gtk_label_set_xalign (GTK_LABEL (label), 0.0);
@@ -468,31 +421,20 @@ _build_section_header (GsUpdatesSection *self)
 	/* use a stack so we can switch which buttons are showing without the
 	 * sizegroup resizing */
 	self->button_stack = GTK_STACK (gtk_stack_new ());
-	gtk_box_pack_end (GTK_BOX (header), GTK_WIDGET (self->button_stack), FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (header), GTK_WIDGET (self->button_stack));
+	gtk_container_child_set (GTK_CONTAINER (header), GTK_WIDGET (self->button_stack), "pack-type", GTK_PACK_END, NULL);
+	gtk_size_group_add_widget (self->sizegroup_button, GTK_WIDGET (self->button_stack));
 
-	/* add download button */
-	self->button_download = gs_progress_button_new ();
-	gtk_button_set_use_underline (GTK_BUTTON (self->button_download), TRUE);
-	gtk_button_set_label (GTK_BUTTON (self->button_download), _("_Download"));
-	context = gtk_widget_get_style_context (self->button_download);
+	/* add button, which may be hidden */
+	self->button = gs_progress_button_new ();
+	context = gtk_widget_get_style_context (self->button);
 	gtk_style_context_add_class (context, GTK_STYLE_CLASS_SUGGESTED_ACTION);
-	g_signal_connect (self->button_download, "clicked",
-			  G_CALLBACK (_button_download_clicked_cb),
-			  self);
-	gtk_stack_add_named (self->button_stack, self->button_download, "download");
-	gtk_widget_set_visible (self->button_download, TRUE);
-	gtk_widget_set_margin_end (self->button_download, 6);
-
-	/* add update button */
-	self->button_update = gs_progress_button_new ();
-	context = gtk_widget_get_style_context (self->button_update);
-	gtk_style_context_add_class (context, GTK_STYLE_CLASS_SUGGESTED_ACTION);
-	g_signal_connect (self->button_update, "clicked",
+	g_signal_connect (self->button, "clicked",
 			  G_CALLBACK (_button_update_all_clicked_cb),
 			  self);
-	gtk_stack_add_named (self->button_stack, self->button_update, "update");
-	gtk_widget_set_visible (self->button_update, TRUE);
-	gtk_widget_set_margin_end (self->button_update, 6);
+	gtk_stack_add_named (self->button_stack, self->button, "update");
+	gtk_widget_set_visible (self->button, TRUE);
+	gtk_widget_set_margin_end (self->button, 6);
 
 	/* add cancel button */
 	self->button_cancel = gs_progress_button_new ();
@@ -504,6 +446,8 @@ _build_section_header (GsUpdatesSection *self)
 	gtk_stack_add_named (self->button_stack, self->button_cancel, "cancel");
 	gtk_widget_set_visible (self->button_cancel, TRUE);
 
+	_update_buttons (self);
+
 	/* success */
 	return header;
 }
@@ -516,10 +460,7 @@ _list_header_func (GtkListBoxRow *row, GtkListBoxRow *before, gpointer user_data
 
 	/* section changed */
 	if (before == NULL) {
-		GtkWidget *parent;
-		if ((parent = gtk_widget_get_parent (GTK_WIDGET (self->section_header))) != NULL)
-			gtk_container_remove (GTK_CONTAINER (parent), GTK_WIDGET (self->section_header));
-		header = self->section_header;
+		header = _build_section_header (self);
 	} else {
 		header = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
 	}
@@ -546,14 +487,6 @@ _app_row_activated_cb (GtkListBox *list_box, GtkListBoxRow *row, GsUpdatesSectio
 				  G_CALLBACK (gtk_widget_destroy), dialog);
 }
 
-static void
-gs_updates_section_show (GtkWidget *widget)
-{
-	_update_buttons (GS_UPDATES_SECTION (widget));
-
-	GTK_WIDGET_CLASS (gs_updates_section_parent_class)->show (widget);
-}
-
 static void
 gs_updates_section_dispose (GObject *object)
 {
@@ -563,16 +496,12 @@ gs_updates_section_dispose (GObject *object)
 	g_clear_object (&self->list);
 	g_clear_object (&self->plugin_loader);
 	g_clear_object (&self->page);
+
 	g_clear_object (&self->sizegroup_image);
 	g_clear_object (&self->sizegroup_name);
 	g_clear_object (&self->sizegroup_desc);
 	g_clear_object (&self->sizegroup_button);
 	g_clear_object (&self->sizegroup_header);
-	self->button_download = NULL;
-	self->button_update = NULL;
-	self->button_cancel = NULL;
-	self->button_stack = NULL;
-	g_clear_object (&self->section_header);
 
 	G_OBJECT_CLASS (gs_updates_section_parent_class)->dispose (object);
 }
@@ -581,10 +510,7 @@ static void
 gs_updates_section_class_init (GsUpdatesSectionClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
-
 	object_class->dispose = gs_updates_section_dispose;
-	widget_class->show = gs_updates_section_show;
 }
 
 void
@@ -600,9 +526,6 @@ gs_updates_section_set_size_groups (GsUpdatesSection *self,
 	g_set_object (&self->sizegroup_desc, desc);
 	g_set_object (&self->sizegroup_button, button);
 	g_set_object (&self->sizegroup_header, header);
-
-	gtk_size_group_add_widget (self->sizegroup_button, GTK_WIDGET (self->button_stack));
-	gtk_size_group_add_widget (self->sizegroup_header, self->section_header);
 }
 
 static void
@@ -657,7 +580,6 @@ gs_updates_section_new (GsUpdatesSectionKind kind,
 	self->kind = kind;
 	self->plugin_loader = g_object_ref (plugin_loader);
 	self->page = g_object_ref (page);
-	self->section_header = g_object_ref_sink (_build_section_header (self));
 	return GTK_LIST_BOX (self);
 }
 
